# -*- coding: utf-8 -*- #
# Copyright 2014 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Resource printer base class.

Each printer has three main attributes, all accessible as strings in the
--format='NAME[ATTRIBUTES](PROJECTION)' option:

  NAME: str, The printer name.

  [ATTRIBUTES]: str, An optional [no-]name[=value] list of attributes. Unknown
    attributes are silently ignored. Attributes are added to a printer local
    dict indexed by name.

  (PROJECTION): str, List of resource names to be included in the output
    resource. Unknown names are silently ignored. Resource names are
    '.'-separated key identifiers with an implicit top level resource name.

Example:

  gcloud compute instances list \
      --format='table[box](name, networkInterfaces[0].networkIP)'
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import io

from googlecloudsdk.core import exceptions as core_exceptions
from googlecloudsdk.core import log
from googlecloudsdk.core.console import console_io
from googlecloudsdk.core.resource import resource_projector
from googlecloudsdk.core.resource import resource_property


# Structured output indentation.
STRUCTURED_INDENTATION = 2


class Error(core_exceptions.Error):
  """Exceptions for this module."""


class ProjectionRequiredError(Error):
  """Format missing required projection exception."""


class _ResourceMarker(object):
  """A marker that can be injected into resource lists."""

  def Act(self, printer):
    """Called by ResourcePrinter.Addrecord().

    Args:
      printer: The printer object.
    """
    pass


class FinishMarker(_ResourceMarker):
  """A resource list Finish marker."""

  def Act(self, printer):
    printer.Finish()


class PageMarker(_ResourceMarker):
  """A resource list Page marker."""

  def Act(self, printer):
    printer.Page()


def IsResourceMarker(resource):
  """Returns True if resource is a _ResourceMarker."""
  return isinstance(resource, _ResourceMarker)


class ResourcePrinter(object):
  """Base class for printing JSON-serializable Python objects.

  Attributes:
    attributes: Optional printer attribute dict indexed by attribute name.
    column_attributes: Projection column attributes.
    _console_attr: The console attributes. May be ignored by some printers.
    _empty: True if there are no records.
    _heading: The list of column heading label strings.
    _name: Format name.
    _non_empty_projection_required: True if the printer requires a non-empty
      projection.
    _out: Output stream.
    _process_record: The function called to process each record passed to
      AddRecord() before calling _AddRecord(). It is called like this:
        record = process_record(record)
    _printer: The resource_printer.Printer method for nested formats.
    _is_column_visible: List of Boolean values indicating if indexed column is
      visible
  """

  def __init__(self, out=None, name=None, projector=None, by_columns=False,
               ignore_default_transforms=False, process_record=None,
               non_empty_projection_required=False, printer=None,
               console_attr=None, retain_none_values=False):
    """Constructor.

    Args:
      out: The output stream, log.out if None. If the 'private' attribute is set
        and the output stream is a log._ConsoleWriter then the underlying stream
        is used instead to disable output to the log file.
      name: The format name.
      projector: Optional resource Projector.
      by_columns: True if AddRecord() expects a list of columns.
      ignore_default_transforms: Ignore default projection transforms if True.
      process_record: The function called to process each record passed to
        AddRecord() before calling _AddRecord(). It is called like this:
          record = process_record(record)
      non_empty_projection_required: True if the printer requires a non-empty
        projection.
      printer: The resource_printer.Printer method for nested formats.
      console_attr: The console attributes for the output stream. Ignored by
        some printers. If None then printers that require it will initialize it
        to match out.
      retain_none_values: Retain resurce dict entries with None values.
    """
    self._console_attr = console_attr
    self._empty = True
    self._heading = None
    self._name = name
    self._non_empty_projection_required = non_empty_projection_required
    self._out = out or log.out
    self._printer = printer
    self._pager = False
    self._pager_out = None

    if not projector:
      projector = resource_projector.Compile()
    self._process_record = process_record or projector.Evaluate
    projector.SetByColumns(by_columns)
    projector.SetIgnoreDefaultTransforms(ignore_default_transforms)
    projection = projector.Projection()
    if projection:
      self.attributes = projection.Attributes() or {}
      self.column_attributes = projection
      if 'debug' in self.attributes:
        log.status.Print('{0} format projection:'.format(projection.Name()))
        projection.Print(log.status)
    else:
      self.attributes = {}
      self.column_attributes = None
    if 'no-undefined' in self.attributes:
      retain_none_values = False
    projector.SetRetainNoneValues(retain_none_values)

    if 'private' in self.attributes:
      try:
        # Disable log file writes by printing directly to the console stream.
        self._out = self._out.GetConsoleWriterStream()
      except AttributeError:
        pass

    if 'pager' in self.attributes and out is None:
      self._pager_out = self._out
      self._out = io.StringIO()
      self._pager = True
    self._SetVisibleColumns()

  def _SetVisibleColumns(self):
    """Sets visible columns list if column attributes have been loaded."""
    if self.column_attributes:
      self._is_column_visible = [
          not column.attribute.hidden
          for column in self.column_attributes.Columns()
      ]
    else:
      self._is_column_visible = None

  def RemoveHiddenColumns(self, row):
    """Returns a list of visible columns given a row."""
    if not self._is_column_visible:
      self._SetVisibleColumns()
    if self._is_column_visible:
      return [col for i, col in enumerate(row) if self._is_column_visible[i]]
    else:
      return row

  def AddHeading(self, heading):
    """Overrides the default heading.

    If the printer does not support headings then this is a no-op.

    Args:
      heading: List of column heading strings that overrides the default
        heading.
    """
    self._heading = heading

  def _AddRecord(self, record, delimit=True):
    """Format specific AddRecord().

    Args:
      record: A JSON-serializable object.
      delimit: Prints resource delimiters if True.
    """
    pass

  def AddRecord(self, record, delimit=True):
    """Adds a record for printing.

    Streaming formats (e.g., YAML) can print results at each AddRecord() call.
    Non-streaming formats (e.g., JSON, table(...)) may cache data at each
    AddRecord() call and not print until Finish() is called.

    Args:
      record: A JSON-serializable object.
      delimit: Prints resource delimiters if True.
    """
    if IsResourceMarker(record):
      record.Act(self)
    else:
      self._empty = False
      self._AddRecord(self._process_record(record), delimit)

  def Finish(self):
    """Prints the results for non-streaming formats.

    Must be called via super if overridden.
    """
    if self._pager and self._out:
      try:
        console_io.More(self._out.getvalue(), out=self._pager_out)
      # Can happen if caller provided a different out that is not type StringIO.
      except AttributeError:
        pass

  def ResourcesWerePrinted(self):
    """Returns True if some resource items were printed or printer disabled."""
    return not self._empty

  def Page(self):
    """Flushes intermediate results for streaming formats."""
    pass

  def PrintSingleRecord(self, record):
    """Print one record by itself.

    Args:
      record: A JSON-serializable object.
    """
    self.AddRecord(record, delimit=False)
    self.Finish()

  def Print(self, resources, single=False, intermediate=False):
    """Prints resources using printer.AddRecord() and printer.Finish().

    Args:
      resources: A singleton or list of JSON-serializable Python objects.
      single: If True then resources is a single item and not a list.
        For example, use this to print a single object as JSON.
      intermediate: This is an intermediate call, do not call Finish().

    Raises:
      ProjectionRequiredError: If the projection is empty and the format
        requires a non-empty projection.
    """
    if 'disable' in self.attributes:
      # Disable formatted output and do not consume the resources.
      self._empty = False
      return
    if self._non_empty_projection_required and (
        not self.column_attributes or not self.column_attributes.Columns()):
      raise ProjectionRequiredError(
          'Format [{0}] requires a non-empty projection. Use key parameters to specify a projection like so `{0}(foo, bar.baz)`'
          .format(self._name))
    # Resources may be a generator and since generators can raise exceptions, we
    # have to call Finish() in the finally block to make sure that the resources
    # we've been able to pull out of the generator are printed before control is
    # given to the exception-handling code.
    try:
      if resources:
        if single or not resource_property.IsListLike(resources):
          self.AddRecord(resources, delimit=intermediate)
        else:
          for resource in resources:
            self.AddRecord(resource)
    finally:
      if not intermediate:
        self.Finish()

  def Printer(self, *args, **kwargs):
    """Calls the resource_printer.Printer() method (for nested printers)."""
    return self._printer(*args, **kwargs)



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Additional flags for data-catalog tags commands."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope.concepts import concepts
from googlecloudsdk.command_lib.util.apis import yaml_data
from googlecloudsdk.command_lib.util.concepts import concept_parsers
from googlecloudsdk.command_lib.util.concepts import presentation_specs


def AddCreateUpdateTagFlags():
  """Hook for adding flags to tags create and update."""
  resource_spec = concepts.ResourceSpec.FromYaml(
      yaml_data.ResourceYAMLData.FromPath('data_catalog.tag_template')
      .GetData())
  presentation_spec = presentation_specs.ResourcePresentationSpec(
      name='--tag-template',
      concept_spec=resource_spec,
      prefixes=True,
      required=True,
      flag_name_overrides={
          'project': '--tag-template-project',
      },
      group_help="""\
Tag template. `--tag-template-location` defaults to the tag's location.
`--tag-template-project` defaults to the tag's project.
      """
  )
  tag_template_arg = concept_parsers.ConceptParser(
      [presentation_spec],
      command_level_fallthroughs={
          '--tag-template.location': ['--location'],
          '--tag-template.project': ['--project'],
      },
  )

  return [tag_template_arg]



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Validate that a terraform plan complies with policies."""
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import os.path

from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.util.anthos import binary_operations
from googlecloudsdk.core import log
from googlecloudsdk.core import metrics
from googlecloudsdk.core import properties
from googlecloudsdk.core.console import progress_tracker
from googlecloudsdk.core.credentials.store import GetFreshAccessToken
from googlecloudsdk.core.util import files

MISSING_BINARY = ('Could not locate terraform-tools executable [{binary}]. '
                  'Please ensure gcloud terraform-tools component is '
                  'properly installed. '
                  'See https://cloud.google.com/sdk/docs/components for '
                  'more details.')


class TerraformToolsTfplanToCaiOperation(
    binary_operations.StreamingBinaryBackedOperation):
  """Streaming operation for Terraform Tools tfplan-to-cai command."""
  custom_errors = {}

  def __init__(self, **kwargs):
    custom_errors = {
        'MISSING_EXEC': MISSING_BINARY.format(binary='terraform-tools'),
    }
    super(TerraformToolsTfplanToCaiOperation, self).__init__(
        binary='terraform-tools',
        check_hidden=True,
        install_if_missing=True,
        custom_errors=custom_errors,
        structured_output=True,
        **kwargs)

  def _ParseArgsForCommand(self, command, terraform_plan_json, project,
                           verbosity, output_path, **kwargs):
    args = [
        command,
        terraform_plan_json,
        '--output-path',
        output_path,
        '--verbosity',
        verbosity,
        '--user-agent',
        metrics.GetUserAgent(),
    ]
    if project:
      args += ['--project', project]
    return args


class TerraformToolsValidateOperation(binary_operations.BinaryBackedOperation):
  """operation for Terraform Tools validate-cai command."""
  custom_errors = {}

  def __init__(self, **kwargs):
    custom_errors = {
        'MISSING_EXEC': MISSING_BINARY.format(binary='terraform-tools'),
    }
    super(TerraformToolsValidateOperation, self).__init__(
        binary='terraform-tools',
        check_hidden=True,
        # Install will be handled by the conversion operation
        install_if_missing=False,
        custom_errors=custom_errors,
        **kwargs)

  def _ParseArgsForCommand(self, command, input_file, policy_library, verbosity,
                           **kwargs):
    args = [
        command,
        input_file,
        '--verbosity',
        verbosity,
        '--policy-library',
        os.path.expanduser(policy_library),
    ]
    return args


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA)
class Vet(base.Command):
  """Validate that a terraform plan complies with policies."""

  detailed_help = {
      'EXAMPLES':
          """
        To validate that a terraform plan complies with a policy library
        at `/my/policy/library`:

        $ {command} tfplan.json --policy-library=/my/policy/library
        """,
  }

  @staticmethod
  def Args(parser):
    parser.add_argument(
        'terraform_plan_json',
        help=(
            'File which contains a JSON export of a terraform plan. This file '
            'will be validated against the given policy library.'),
    )
    parser.add_argument(
        '--policy-library',
        required=True,
        help='Directory which contains a policy library',
    )

  def Run(self, args):
    tfplan_to_cai_operation = TerraformToolsTfplanToCaiOperation()
    validate_cai_operation = TerraformToolsValidateOperation()
    validate_tfplan_operation = TerraformToolsValidateOperation()

    env_vars = {
        'GOOGLE_OAUTH_ACCESS_TOKEN':
            GetFreshAccessToken(account=properties.VALUES.core.account.Get()),
        'USE_STRUCTURED_LOGGING':
            'true',
    }

    with files.TemporaryDirectory() as tempdir:
      cai_assets = os.path.join(tempdir, 'cai_assets.json')

      response = tfplan_to_cai_operation(
          command='tfplan-to-cai',
          project=args.project or properties.VALUES.core.project.Get(),
          terraform_plan_json=args.terraform_plan_json,
          verbosity=args.verbosity,
          output_path=cai_assets,
          env=env_vars)
      self.exit_code = response.exit_code
      if self.exit_code > 0:
        # The streaming binary backed operation handles its own writing to
        # stdout and stderr, so there's nothing left to do here.
        return None

      with progress_tracker.ProgressTracker(
          message='Validating resources',
          aborted_message='Aborted validation.'):
        cai_response = validate_cai_operation(
            command='validate-cai',
            policy_library=args.policy_library,
            input_file=cai_assets,
            verbosity=args.verbosity,
            env=env_vars)
        tfplan_response = validate_tfplan_operation(
            command='validate-tfplan',
            policy_library=args.policy_library,
            input_file=args.terraform_plan_json,
            verbosity=args.verbosity,
            env=env_vars)

    # exit code 2 from a validate_* command indicates violations; we need to
    # pass that through to users so they can detect this case. However, if
    # either command errors out (exit code 1) return that instead.
    if cai_response.exit_code == 1 or tfplan_response.exit_code == 1:
      self.exit_code = 1
    elif cai_response.exit_code == 2 or tfplan_response.exit_code == 2:
      self.exit_code = 2

    # Output from validate commands uses "structured output", same as the
    # streaming output from conversion. The final output should be a combined
    # list of violations.
    violations = []

    for policy_type, response in (('CAI', cai_response), ('Terraform',
                                                          tfplan_response)):
      if response.stdout:
        try:
          msg = binary_operations.ReadStructuredOutput(
              response.stdout, as_json=True)
        except binary_operations.StructuredOutputError:
          log.warning('Could not parse {} policy validation output.'.format(
              policy_type))
        else:
          violations += msg.resource_body
      if response.stderr:
        handler = binary_operations.DefaultStreamStructuredErrHandler(None)
        for line in response.stderr.split('\n'):
          handler(line)

    return violations



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Utilities for using containers in conjunction with ssh."""


from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals


def GetRemoteCommand(container, command):
  """Assemble the remote command list given user-supplied args.

  If a container argument is supplied, run
  `sudo docker exec -i[t] CONTAINER_ID COMMAND [ARGS...]` on the remote.

  Args:
    container: str or None, name of container to enter during connection.
    command: [str] or None, the remote command to execute. If no command is
      given, allocate a TTY.

  Returns:
    [str] or None, Remote command to run or None if no command.
  """
  if container:
    # The `/bin/sh` is for listening to commands.
    args = command or ['/bin/sh']
    flags = '-i' if command else '-it'
    return ['sudo', 'docker', 'exec', flags, container] + args
  if command:
    return command
  return None


def GetTty(container, command):
  """Determine the ssh command should be run in a TTY or not.

  Args:
    container: str or None, name of container to enter during connection.
    command: [str] or None, the remote command to execute. If no command is
      given, allocate a TTY.

  Returns:
    Bool or None, whether to enforce TTY or not, or None if "auto".
  """
  return True if container and not command else None




"""Generated message classes for baremetalsolution version v2.

Provides ways to manage Bare Metal Solution hardware installed in a regional
extension located near a Google Cloud data center.
"""
# NOTE: This file is autogenerated and should not be edited by hand.

from __future__ import absolute_import

from apitools.base.protorpclite import messages as _messages
from apitools.base.py import encoding
from apitools.base.py import extra_types


package = 'baremetalsolution'


class AllocateLunsRequest(_messages.Message):
  r"""Message for creating Luns for Volume.

  Fields:
    lunRanges: Required. LUN ranges to be allocated.
  """

  lunRanges = _messages.MessageField('AllocateLunsRequestLunRange', 1, repeated=True)


class AllocateLunsRequestLunRange(_messages.Message):
  r"""A LUN(Logical Unit Number) range.

  Fields:
    quantity: Number of LUNs to create.
    sizeGb: The requested size of each LUN, in GB.
  """

  quantity = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  sizeGb = _messages.IntegerField(2, variant=_messages.Variant.INT32)


class AllowedClient(_messages.Message):
  r"""Represents an 'access point' for the share.

  Enums:
    MountPermissionsValueValuesEnum: Mount permissions.

  Fields:
    allowDev: Allow dev flag. Which controls whether to allow creation of
      devices.
    allowSuid: Allow the setuid flag.
    allowedClientsCidr: The subnet of IP addresses permitted to access the
      share.
    mountPermissions: Mount permissions.
    network: The network the access point sits on.
    nfsPath: Output only. The path to access NFS, in format
      shareIP:/InstanceID InstanceID is the generated ID instead of customer
      provided name. example like "10.0.0.0:/g123456789-nfs001"
    noRootSquash: Disable root squashing, which is a feature of NFS. Root
      squash is a special mapping of the remote superuser (root) identity when
      using identity authentication.
    shareIp: Output only. The IP address of the share on this network.
      Assigned automatically during provisioning based on the network's
      services_cidr.
  """

  class MountPermissionsValueValuesEnum(_messages.Enum):
    r"""Mount permissions.

    Values:
      MOUNT_PERMISSIONS_UNSPECIFIED: Permissions were not specified.
      READ: NFS share can be mount with read-only permissions.
      READ_WRITE: NFS share can be mount with read-write permissions.
    """
    MOUNT_PERMISSIONS_UNSPECIFIED = 0
    READ = 1
    READ_WRITE = 2

  allowDev = _messages.BooleanField(1)
  allowSuid = _messages.BooleanField(2)
  allowedClientsCidr = _messages.StringField(3)
  mountPermissions = _messages.EnumField('MountPermissionsValueValuesEnum', 4)
  network = _messages.StringField(5)
  nfsPath = _messages.StringField(6)
  noRootSquash = _messages.BooleanField(7)
  shareIp = _messages.StringField(8)


class AttachVolumeRequest(_messages.Message):
  r"""Message for attaching Volume to an instance. All Luns of the Volume will
  be attached.

  Fields:
    volume: Required. Name of the Volume to attach.
  """

  volume = _messages.StringField(1)


class BaremetalsolutionProjectsLocationsGetRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsGetRequest object.

  Fields:
    name: Resource name for the location.
  """

  name = _messages.StringField(1, required=True)


class BaremetalsolutionProjectsLocationsInstanceProvisioningSettingsFetchRequest(_messages.Message):
  r"""A
  BaremetalsolutionProjectsLocationsInstanceProvisioningSettingsFetchRequest
  object.

  Fields:
    location: Required. The parent project and location containing the
      ProvisioningSettings.
  """

  location = _messages.StringField(1, required=True)


class BaremetalsolutionProjectsLocationsInstanceQuotasListRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsInstanceQuotasListRequest object.

  Fields:
    pageSize: Requested page size. The server might return fewer items than
      requested. If unspecified, server will pick an appropriate default.
      Notice that page_size field is not supported and won't be respected in
      the API request for now, will be updated when pagination is supported.
    pageToken: A token identifying a page of results from the server.
    parent: Required. Parent value for ListProvisioningQuotasRequest.
  """

  pageSize = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class BaremetalsolutionProjectsLocationsInstancesAttachVolumeRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsInstancesAttachVolumeRequest object.

  Fields:
    attachVolumeRequest: A AttachVolumeRequest resource to be passed as the
      request body.
    instance: Required. Name of the instance.
  """

  attachVolumeRequest = _messages.MessageField('AttachVolumeRequest', 1)
  instance = _messages.StringField(2, required=True)


class BaremetalsolutionProjectsLocationsInstancesCreateRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsInstancesCreateRequest object.

  Fields:
    instance: A Instance resource to be passed as the request body.
    parent: Required. The parent project and location.
  """

  instance = _messages.MessageField('Instance', 1)
  parent = _messages.StringField(2, required=True)


class BaremetalsolutionProjectsLocationsInstancesDetachLunRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsInstancesDetachLunRequest object.

  Fields:
    detachLunRequest: A DetachLunRequest resource to be passed as the request
      body.
    instance: Required. Name of the instance.
  """

  detachLunRequest = _messages.MessageField('DetachLunRequest', 1)
  instance = _messages.StringField(2, required=True)


class BaremetalsolutionProjectsLocationsInstancesDetachVolumeRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsInstancesDetachVolumeRequest object.

  Fields:
    detachVolumeRequest: A DetachVolumeRequest resource to be passed as the
      request body.
    instance: Required. Name of the instance.
  """

  detachVolumeRequest = _messages.MessageField('DetachVolumeRequest', 1)
  instance = _messages.StringField(2, required=True)


class BaremetalsolutionProjectsLocationsInstancesDisableInteractiveSerialConsoleRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsInstancesDisableInteractiveSerialCon
  soleRequest object.

  Fields:
    disableInteractiveSerialConsoleRequest: A
      DisableInteractiveSerialConsoleRequest resource to be passed as the
      request body.
    name: Required. Name of the resource.
  """

  disableInteractiveSerialConsoleRequest = _messages.MessageField('DisableInteractiveSerialConsoleRequest', 1)
  name = _messages.StringField(2, required=True)


class BaremetalsolutionProjectsLocationsInstancesEnableInteractiveSerialConsoleRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsInstancesEnableInteractiveSerialCons
  oleRequest object.

  Fields:
    enableInteractiveSerialConsoleRequest: A
      EnableInteractiveSerialConsoleRequest resource to be passed as the
      request body.
    name: Required. Name of the resource.
  """

  enableInteractiveSerialConsoleRequest = _messages.MessageField('EnableInteractiveSerialConsoleRequest', 1)
  name = _messages.StringField(2, required=True)


class BaremetalsolutionProjectsLocationsInstancesGetRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsInstancesGetRequest object.

  Fields:
    name: Required. Name of the resource.
  """

  name = _messages.StringField(1, required=True)


class BaremetalsolutionProjectsLocationsInstancesListRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsInstancesListRequest object.

  Fields:
    filter: List filter.
    pageSize: Requested page size. Server may return fewer items than
      requested. If unspecified, the server will pick an appropriate default.
    pageToken: A token identifying a page of results from the server.
    parent: Required. Parent value for ListInstancesRequest.
  """

  filter = _messages.StringField(1)
  pageSize = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(3)
  parent = _messages.StringField(4, required=True)


class BaremetalsolutionProjectsLocationsInstancesPatchRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsInstancesPatchRequest object.

  Fields:
    instance: A Instance resource to be passed as the request body.
    name: Immutable. The resource name of this `Instance`. Resource names are
      schemeless URIs that follow the conventions in
      https://cloud.google.com/apis/design/resource_names. Format:
      `projects/{project}/locations/{location}/instances/{instance}`
    updateMask: The list of fields to update. The currently supported fields
      are: `labels` `hyperthreading_enabled` `os_image`
  """

  instance = _messages.MessageField('Instance', 1)
  name = _messages.StringField(2, required=True)
  updateMask = _messages.StringField(3)


class BaremetalsolutionProjectsLocationsInstancesResetRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsInstancesResetRequest object.

  Fields:
    name: Required. Name of the resource.
    resetInstanceRequest: A ResetInstanceRequest resource to be passed as the
      request body.
  """

  name = _messages.StringField(1, required=True)
  resetInstanceRequest = _messages.MessageField('ResetInstanceRequest', 2)


class BaremetalsolutionProjectsLocationsInstancesStartRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsInstancesStartRequest object.

  Fields:
    name: Required. Name of the resource.
    startInstanceRequest: A StartInstanceRequest resource to be passed as the
      request body.
  """

  name = _messages.StringField(1, required=True)
  startInstanceRequest = _messages.MessageField('StartInstanceRequest', 2)


class BaremetalsolutionProjectsLocationsInstancesStopRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsInstancesStopRequest object.

  Fields:
    name: Required. Name of the resource.
    stopInstanceRequest: A StopInstanceRequest resource to be passed as the
      request body.
  """

  name = _messages.StringField(1, required=True)
  stopInstanceRequest = _messages.MessageField('StopInstanceRequest', 2)


class BaremetalsolutionProjectsLocationsListRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsListRequest object.

  Fields:
    filter: A filter to narrow down results to a preferred subset. The
      filtering language accepts strings like `"displayName=tokyo"`, and is
      documented in more detail in [AIP-160](https://google.aip.dev/160).
    name: The resource that owns the locations collection, if applicable.
    pageSize: The maximum number of results to return. If not set, the service
      selects a default.
    pageToken: A page token received from the `next_page_token` field in the
      response. Send that page token to receive the subsequent page.
  """

  filter = _messages.StringField(1)
  name = _messages.StringField(2, required=True)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)


class BaremetalsolutionProjectsLocationsNetworkQuotasListRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsNetworkQuotasListRequest object.

  Fields:
    pageSize: Requested page size. The server might return fewer items than
      requested. If unspecified, server will pick an appropriate default.
      Notice that page_size field is not supported and won't be respected in
      the API request for now, will be updated when pagination is supported.
    pageToken: A token identifying a page of results from the server.
    parent: Required. Parent value for ListProvisioningQuotasRequest.
  """

  pageSize = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class BaremetalsolutionProjectsLocationsNetworksGetRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsNetworksGetRequest object.

  Fields:
    name: Required. Name of the resource.
  """

  name = _messages.StringField(1, required=True)


class BaremetalsolutionProjectsLocationsNetworksListNetworkUsageRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsNetworksListNetworkUsageRequest
  object.

  Fields:
    location: Required. Parent value (project and location).
  """

  location = _messages.StringField(1, required=True)


class BaremetalsolutionProjectsLocationsNetworksListRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsNetworksListRequest object.

  Fields:
    filter: List filter.
    pageSize: Requested page size. The server might return fewer items than
      requested. If unspecified, server will pick an appropriate default.
    pageToken: A token identifying a page of results from the server.
    parent: Required. Parent value for ListNetworksRequest.
  """

  filter = _messages.StringField(1)
  pageSize = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(3)
  parent = _messages.StringField(4, required=True)


class BaremetalsolutionProjectsLocationsNetworksPatchRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsNetworksPatchRequest object.

  Fields:
    name: Output only. The resource name of this `Network`. Resource names are
      schemeless URIs that follow the conventions in
      https://cloud.google.com/apis/design/resource_names. Format:
      `projects/{project}/locations/{location}/networks/{network}`
    network: A Network resource to be passed as the request body.
    updateMask: The list of fields to update. The only currently supported
      fields are: `labels`, `reservations`, `vrf.vlan_attachments`
  """

  name = _messages.StringField(1, required=True)
  network = _messages.MessageField('Network', 2)
  updateMask = _messages.StringField(3)


class BaremetalsolutionProjectsLocationsNfsSharesCreateRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsNfsSharesCreateRequest object.

  Fields:
    nfsShare: A NfsShare resource to be passed as the request body.
    parent: Required. The parent project and location.
  """

  nfsShare = _messages.MessageField('NfsShare', 1)
  parent = _messages.StringField(2, required=True)


class BaremetalsolutionProjectsLocationsNfsSharesDeleteRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsNfsSharesDeleteRequest object.

  Fields:
    name: Required. The name of the NFS share to delete.
  """

  name = _messages.StringField(1, required=True)


class BaremetalsolutionProjectsLocationsNfsSharesGetRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsNfsSharesGetRequest object.

  Fields:
    name: Required. Name of the resource.
  """

  name = _messages.StringField(1, required=True)


class BaremetalsolutionProjectsLocationsNfsSharesListRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsNfsSharesListRequest object.

  Fields:
    filter: List filter.
    pageSize: Requested page size. The server might return fewer items than
      requested. If unspecified, server will pick an appropriate default.
    pageToken: A token identifying a page of results from the server.
    parent: Required. Parent value for ListNfsSharesRequest.
  """

  filter = _messages.StringField(1)
  pageSize = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(3)
  parent = _messages.StringField(4, required=True)


class BaremetalsolutionProjectsLocationsNfsSharesPatchRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsNfsSharesPatchRequest object.

  Fields:
    name: Immutable. The name of the NFS share.
    nfsShare: A NfsShare resource to be passed as the request body.
    updateMask: The list of fields to update. The only currently supported
      fields are: `labels` `allowed_clients`
  """

  name = _messages.StringField(1, required=True)
  nfsShare = _messages.MessageField('NfsShare', 2)
  updateMask = _messages.StringField(3)


class BaremetalsolutionProjectsLocationsOperationsGetRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsOperationsGetRequest object.

  Fields:
    name: The name of the operation resource.
  """

  name = _messages.StringField(1, required=True)


class BaremetalsolutionProjectsLocationsProvisioningConfigsCreateRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsProvisioningConfigsCreateRequest
  object.

  Fields:
    email: Optional. Email provided to send a confirmation with provisioning
      config to.
    parent: Required. The parent project and location containing the
      ProvisioningConfig.
    provisioningConfig: A ProvisioningConfig resource to be passed as the
      request body.
  """

  email = _messages.StringField(1)
  parent = _messages.StringField(2, required=True)
  provisioningConfig = _messages.MessageField('ProvisioningConfig', 3)


class BaremetalsolutionProjectsLocationsProvisioningConfigsGetRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsProvisioningConfigsGetRequest
  object.

  Fields:
    name: Required. Name of the ProvisioningConfig.
  """

  name = _messages.StringField(1, required=True)


class BaremetalsolutionProjectsLocationsProvisioningConfigsPatchRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsProvisioningConfigsPatchRequest
  object.

  Fields:
    email: Optional. Email provided to send a confirmation with provisioning
      config to.
    name: Output only. The system-generated name of the provisioning config.
      This follows the UUID format.
    provisioningConfig: A ProvisioningConfig resource to be passed as the
      request body.
    updateMask: Required. The list of fields to update.
  """

  email = _messages.StringField(1)
  name = _messages.StringField(2, required=True)
  provisioningConfig = _messages.MessageField('ProvisioningConfig', 3)
  updateMask = _messages.StringField(4)


class BaremetalsolutionProjectsLocationsProvisioningConfigsSubmitRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsProvisioningConfigsSubmitRequest
  object.

  Fields:
    parent: Required. The parent project and location containing the
      ProvisioningConfig.
    submitProvisioningConfigRequest: A SubmitProvisioningConfigRequest
      resource to be passed as the request body.
  """

  parent = _messages.StringField(1, required=True)
  submitProvisioningConfigRequest = _messages.MessageField('SubmitProvisioningConfigRequest', 2)


class BaremetalsolutionProjectsLocationsProvisioningQuotasListRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsProvisioningQuotasListRequest
  object.

  Fields:
    pageSize: Requested page size. The server might return fewer items than
      requested. If unspecified, server will pick an appropriate default.
      Notice that page_size field is not supported and won't be respected in
      the API request for now, will be updated when pagination is supported.
    pageToken: A token identifying a page of results from the server.
    parent: Required. Parent value for ListProvisioningQuotasRequest.
  """

  pageSize = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class BaremetalsolutionProjectsLocationsSnapshotSchedulePoliciesCreateRequest(_messages.Message):
  r"""A
  BaremetalsolutionProjectsLocationsSnapshotSchedulePoliciesCreateRequest
  object.

  Fields:
    parent: Required. The parent project and location containing the
      SnapshotSchedulePolicy.
    snapshotSchedulePolicy: A SnapshotSchedulePolicy resource to be passed as
      the request body.
    snapshotSchedulePolicyId: Required. Snapshot policy ID
  """

  parent = _messages.StringField(1, required=True)
  snapshotSchedulePolicy = _messages.MessageField('SnapshotSchedulePolicy', 2)
  snapshotSchedulePolicyId = _messages.StringField(3)


class BaremetalsolutionProjectsLocationsSnapshotSchedulePoliciesDeleteRequest(_messages.Message):
  r"""A
  BaremetalsolutionProjectsLocationsSnapshotSchedulePoliciesDeleteRequest
  object.

  Fields:
    name: Required. The name of the snapshot schedule policy to delete.
  """

  name = _messages.StringField(1, required=True)


class BaremetalsolutionProjectsLocationsSnapshotSchedulePoliciesGetRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsSnapshotSchedulePoliciesGetRequest
  object.

  Fields:
    name: Required. Name of the resource.
  """

  name = _messages.StringField(1, required=True)


class BaremetalsolutionProjectsLocationsSnapshotSchedulePoliciesListRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsSnapshotSchedulePoliciesListRequest
  object.

  Fields:
    filter: List filter.
    pageSize: The maximum number of items to return.
    pageToken: The next_page_token value returned from a previous List
      request, if any.
    parent: Required. The parent project containing the Snapshot Schedule
      Policies.
  """

  filter = _messages.StringField(1)
  pageSize = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(3)
  parent = _messages.StringField(4, required=True)


class BaremetalsolutionProjectsLocationsSnapshotSchedulePoliciesPatchRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsSnapshotSchedulePoliciesPatchRequest
  object.

  Fields:
    name: Output only. The name of the snapshot schedule policy.
    snapshotSchedulePolicy: A SnapshotSchedulePolicy resource to be passed as
      the request body.
    updateMask: Required. The list of fields to update.
  """

  name = _messages.StringField(1, required=True)
  snapshotSchedulePolicy = _messages.MessageField('SnapshotSchedulePolicy', 2)
  updateMask = _messages.StringField(3)


class BaremetalsolutionProjectsLocationsSshKeysCreateRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsSshKeysCreateRequest object.

  Fields:
    parent: Required. The parent containing the SSH keys.
    sSHKey: A SSHKey resource to be passed as the request body.
    sshKeyId: Required. The ID to use for the key, which will become the final
      component of the key's resource name. This value must match the regex:
      [a-zA-Z0-9@.\-_]{1,64}
  """

  parent = _messages.StringField(1, required=True)
  sSHKey = _messages.MessageField('SSHKey', 2)
  sshKeyId = _messages.StringField(3)


class BaremetalsolutionProjectsLocationsSshKeysDeleteRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsSshKeysDeleteRequest object.

  Fields:
    name: Required. The name of the SSH key to delete. Currently, the only
      valid value for the location is "global".
  """

  name = _messages.StringField(1, required=True)


class BaremetalsolutionProjectsLocationsSshKeysListRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsSshKeysListRequest object.

  Fields:
    pageSize: The maximum number of items to return.
    pageToken: The next_page_token value returned from a previous List
      request, if any.
    parent: Required. The parent containing the SSH keys. Currently, the only
      valid value for the location is "global".
  """

  pageSize = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class BaremetalsolutionProjectsLocationsStorageQuotasListRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsStorageQuotasListRequest object.

  Fields:
    pageSize: Requested page size. The server might return fewer items than
      requested. If unspecified, server will pick an appropriate default.
      Notice that page_size field is not supported and won't be respected in
      the API request for now, will be updated when pagination is supported.
    pageToken: A token identifying a page of results from the server.
    parent: Required. Parent value for ListProvisioningQuotasRequest.
  """

  pageSize = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class BaremetalsolutionProjectsLocationsVolumesAllocateLunsRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsVolumesAllocateLunsRequest object.

  Fields:
    allocateLunsRequest: A AllocateLunsRequest resource to be passed as the
      request body.
    parent: Required. Parent volume.
  """

  allocateLunsRequest = _messages.MessageField('AllocateLunsRequest', 1)
  parent = _messages.StringField(2, required=True)


class BaremetalsolutionProjectsLocationsVolumesCreateAndAttachRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsVolumesCreateAndAttachRequest
  object.

  Fields:
    createAndAttachVolumeRequest: A CreateAndAttachVolumeRequest resource to
      be passed as the request body.
    parent: Required. The parent project and location.
  """

  createAndAttachVolumeRequest = _messages.MessageField('CreateAndAttachVolumeRequest', 1)
  parent = _messages.StringField(2, required=True)


class BaremetalsolutionProjectsLocationsVolumesCreateRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsVolumesCreateRequest object.

  Fields:
    parent: Required. The parent project and location.
    volume: A Volume resource to be passed as the request body.
  """

  parent = _messages.StringField(1, required=True)
  volume = _messages.MessageField('Volume', 2)


class BaremetalsolutionProjectsLocationsVolumesDeleteRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsVolumesDeleteRequest object.

  Fields:
    name: Required. The name of the Volume to delete.
  """

  name = _messages.StringField(1, required=True)


class BaremetalsolutionProjectsLocationsVolumesGetRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsVolumesGetRequest object.

  Fields:
    name: Required. Name of the resource.
  """

  name = _messages.StringField(1, required=True)


class BaremetalsolutionProjectsLocationsVolumesListRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsVolumesListRequest object.

  Fields:
    filter: List filter.
    pageSize: Requested page size. The server might return fewer items than
      requested. If unspecified, server will pick an appropriate default.
    pageToken: A token identifying a page of results from the server.
    parent: Required. Parent value for ListVolumesRequest.
  """

  filter = _messages.StringField(1)
  pageSize = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(3)
  parent = _messages.StringField(4, required=True)


class BaremetalsolutionProjectsLocationsVolumesLunsDeleteRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsVolumesLunsDeleteRequest object.

  Fields:
    name: Required. The name of the Lun to delete.
  """

  name = _messages.StringField(1, required=True)


class BaremetalsolutionProjectsLocationsVolumesLunsGetRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsVolumesLunsGetRequest object.

  Fields:
    name: Required. Name of the resource.
  """

  name = _messages.StringField(1, required=True)


class BaremetalsolutionProjectsLocationsVolumesLunsListRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsVolumesLunsListRequest object.

  Fields:
    pageSize: Requested page size. The server might return fewer items than
      requested. If unspecified, server will pick an appropriate default.
    pageToken: A token identifying a page of results from the server.
    parent: Required. Parent value for ListLunsRequest.
  """

  pageSize = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class BaremetalsolutionProjectsLocationsVolumesPatchRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsVolumesPatchRequest object.

  Fields:
    name: Output only. The resource name of this `Volume`. Resource names are
      schemeless URIs that follow the conventions in
      https://cloud.google.com/apis/design/resource_names. Format:
      `projects/{project}/locations/{location}/volumes/{volume}`
    updateMask: The list of fields to update. The only currently supported
      fields are: 'labels'
    volume: A Volume resource to be passed as the request body.
  """

  name = _messages.StringField(1, required=True)
  updateMask = _messages.StringField(2)
  volume = _messages.MessageField('Volume', 3)


class BaremetalsolutionProjectsLocationsVolumesResizeRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsVolumesResizeRequest object.

  Fields:
    resizeVolumeRequest: A ResizeVolumeRequest resource to be passed as the
      request body.
    volume: Required. Volume to resize.
  """

  resizeVolumeRequest = _messages.MessageField('ResizeVolumeRequest', 1)
  volume = _messages.StringField(2, required=True)


class BaremetalsolutionProjectsLocationsVolumesSnapshotsCreateRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsVolumesSnapshotsCreateRequest
  object.

  Fields:
    parent: Required. The volume to snapshot.
    volumeSnapshot: A VolumeSnapshot resource to be passed as the request
      body.
  """

  parent = _messages.StringField(1, required=True)
  volumeSnapshot = _messages.MessageField('VolumeSnapshot', 2)


class BaremetalsolutionProjectsLocationsVolumesSnapshotsDeleteRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsVolumesSnapshotsDeleteRequest
  object.

  Fields:
    name: Required. The name of the snapshot to delete.
  """

  name = _messages.StringField(1, required=True)


class BaremetalsolutionProjectsLocationsVolumesSnapshotsGetRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsVolumesSnapshotsGetRequest object.

  Fields:
    name: Required. The name of the snapshot.
  """

  name = _messages.StringField(1, required=True)


class BaremetalsolutionProjectsLocationsVolumesSnapshotsListRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsVolumesSnapshotsListRequest object.

  Fields:
    pageSize: Requested page size. The server might return fewer items than
      requested. If unspecified, server will pick an appropriate default.
    pageToken: A token identifying a page of results from the server.
    parent: Required. Parent value for ListVolumesRequest.
  """

  pageSize = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class BaremetalsolutionProjectsLocationsVolumesSnapshotsRestoreVolumeSnapshotRequest(_messages.Message):
  r"""A BaremetalsolutionProjectsLocationsVolumesSnapshotsRestoreVolumeSnapsho
  tRequest object.

  Fields:
    restoreVolumeSnapshotRequest: A RestoreVolumeSnapshotRequest resource to
      be passed as the request body.
    volumeSnapshot: Required. Name of the snapshot which will be used to
      restore its parent volume.
  """

  restoreVolumeSnapshotRequest = _messages.MessageField('RestoreVolumeSnapshotRequest', 1)
  volumeSnapshot = _messages.StringField(2, required=True)


class CreateAndAttachVolumeRequest(_messages.Message):
  r"""Message for creating a volume with immediate Luns allocation and their
  attachment to instances.

  Fields:
    instances: List of instance to attach this volume to. If defined, will
      attach all LUNs of this Volume to specified instances. Makes sense only
      when lun_ranges are defined.
    lunRanges: LUN ranges to be allocated. If defined, will immediately
      allocate LUNs.
    volume: Required. The volume to create.
  """

  instances = _messages.StringField(1, repeated=True)
  lunRanges = _messages.MessageField('VolumeLunRange', 2, repeated=True)
  volume = _messages.MessageField('Volume', 3)


class DetachLunRequest(_messages.Message):
  r"""Message for detach specific LUN from an Instance.

  Fields:
    lun: Required. Name of the Lun to detach.
    skipReboot: If true, performs lun unmapping without instance reboot.
  """

  lun = _messages.StringField(1)
  skipReboot = _messages.BooleanField(2)


class DetachVolumeRequest(_messages.Message):
  r"""Message for detaching Volume from an instance. All Luns of the Volume
  will be detached.

  Fields:
    skipReboot: If true, performs Volume unmapping without instance reboot.
    volume: Required. Name of the Volume to detach.
  """

  skipReboot = _messages.BooleanField(1)
  volume = _messages.StringField(2)


class DisableInteractiveSerialConsoleRequest(_messages.Message):
  r"""Message for disabling the interactive serial console on an instance."""


class Empty(_messages.Message):
  r"""A generic empty message that you can re-use to avoid defining duplicated
  empty messages in your APIs. A typical example is to use it as the request
  or the response type of an API method. For instance: service Foo { rpc
  Bar(google.protobuf.Empty) returns (google.protobuf.Empty); }
  """



class EnableInteractiveSerialConsoleRequest(_messages.Message):
  r"""Message for enabling the interactive serial console on an instance."""


class FetchInstanceProvisioningSettingsResponse(_messages.Message):
  r"""Response with all provisioning settings.

  Fields:
    images: The OS images available.
  """

  images = _messages.MessageField('OSImage', 1, repeated=True)


class GoogleCloudBaremetalsolutionV2LogicalInterface(_messages.Message):
  r"""Each logical interface represents a logical abstraction of the
  underlying physical interface (for eg. bond, nic) of the instance. Each
  logical interface can effectively map to multiple network-IP pairs and still
  be mapped to one underlying physical interface.

  Fields:
    interfaceIndex: The index of the logical interface mapping to the index of
      the hardware bond or nic on the chosen network template. This field is
      deprecated.
    logicalNetworkInterfaces: List of logical network interfaces within a
      logical interface.
    name: Interface name. This is of syntax or and forms part of the network
      template name.
  """

  interfaceIndex = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  logicalNetworkInterfaces = _messages.MessageField('LogicalNetworkInterface', 2, repeated=True)
  name = _messages.StringField(3)


class GoogleCloudBaremetalsolutionV2ServerNetworkTemplateLogicalInterface(_messages.Message):
  r"""Logical interface.

  Enums:
    TypeValueValuesEnum: Interface type.

  Fields:
    name: Interface name. This is not a globally unique identifier. Name is
      unique only inside the ServerNetworkTemplate. This is of syntax or and
      forms part of the network template name.
    required: If true, interface must have network connected.
    type: Interface type.
  """

  class TypeValueValuesEnum(_messages.Enum):
    r"""Interface type.

    Values:
      INTERFACE_TYPE_UNSPECIFIED: Unspecified value.
      BOND: Bond interface type.
      NIC: NIC interface type.
    """
    INTERFACE_TYPE_UNSPECIFIED = 0
    BOND = 1
    NIC = 2

  name = _messages.StringField(1)
  required = _messages.BooleanField(2)
  type = _messages.EnumField('TypeValueValuesEnum', 3)


class Instance(_messages.Message):
  r"""A server.

  Enums:
    StateValueValuesEnum: Output only. The state of the server.

  Messages:
    LabelsValue: Labels as key value pairs.

  Fields:
    createTime: Output only. Create a time stamp.
    hyperthreadingEnabled: True if you enable hyperthreading for the server,
      otherwise false. The default value is false.
    id: Output only. An identifier for the `Instance`, generated by the
      backend.
    interactiveSerialConsoleEnabled: Output only. True if the interactive
      serial console feature is enabled for the instance, false otherwise. The
      default value is false.
    labels: Labels as key value pairs.
    logicalInterfaces: List of logical interfaces for the instance. The number
      of logical interfaces will be the same as number of hardware bond/nic on
      the chosen network template. For the non-multivlan configurations (for
      eg, existing servers) that use existing default network template
      (bondaa-bondaa), both the Instance.networks field and the
      Instance.logical_interfaces fields will be filled to ensure backward
      compatibility. For the others, only Instance.logical_interfaces will be
      filled.
    loginInfo: Output only. Text field about info for logging in.
    luns: Immutable. List of LUNs associated with this server.
    machineType: Immutable. The server type. [Available server
      types](https://cloud.google.com/bare-metal/docs/bms-
      planning#server_configurations)
    name: Immutable. The resource name of this `Instance`. Resource names are
      schemeless URIs that follow the conventions in
      https://cloud.google.com/apis/design/resource_names. Format:
      `projects/{project}/locations/{location}/instances/{instance}`
    networkTemplate: Instance network template name. For eg, bondaa-bondaa,
      bondab-nic, etc. Generally, the template name follows the syntax of
      "bond" or "nic".
    networks: Output only. List of networks associated with this server.
    osImage: The OS image currently installed on the server.
    pod: Immutable. Pod name. Pod is an independent part of infrastructure.
      Instance can be connected to the assets (networks, volumes) allocated in
      the same pod only.
    state: Output only. The state of the server.
    updateTime: Output only. Update a time stamp.
    volumes: Input only. List of Volumes to attach to this Instance on
      creation. This field won't be populated in Get/List responses.
  """

  class StateValueValuesEnum(_messages.Enum):
    r"""Output only. The state of the server.

    Values:
      STATE_UNSPECIFIED: The server is in an unknown state.
      PROVISIONING: The server is being provisioned.
      RUNNING: The server is running.
      DELETED: The server has been deleted.
      UPDATING: The server is being updated.
    """
    STATE_UNSPECIFIED = 0
    PROVISIONING = 1
    RUNNING = 2
    DELETED = 3
    UPDATING = 4

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Labels as key value pairs.

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  createTime = _messages.StringField(1)
  hyperthreadingEnabled = _messages.BooleanField(2)
  id = _messages.StringField(3)
  interactiveSerialConsoleEnabled = _messages.BooleanField(4)
  labels = _messages.MessageField('LabelsValue', 5)
  logicalInterfaces = _messages.MessageField('GoogleCloudBaremetalsolutionV2LogicalInterface', 6, repeated=True)
  loginInfo = _messages.StringField(7)
  luns = _messages.MessageField('Lun', 8, repeated=True)
  machineType = _messages.StringField(9)
  name = _messages.StringField(10)
  networkTemplate = _messages.StringField(11)
  networks = _messages.MessageField('Network', 12, repeated=True)
  osImage = _messages.StringField(13)
  pod = _messages.StringField(14)
  state = _messages.EnumField('StateValueValuesEnum', 15)
  updateTime = _messages.StringField(16)
  volumes = _messages.MessageField('Volume', 17, repeated=True)


class InstanceConfig(_messages.Message):
  r"""Configuration parameters for a new instance.

  Enums:
    NetworkConfigValueValuesEnum: The type of network configuration on the
      instance.

  Fields:
    accountNetworksEnabled: If true networks can be from different projects of
      the same vendor account.
    clientNetwork: Client network address. Filled if
      InstanceConfig.multivlan_config is false.
    hyperthreading: Whether the instance should be provisioned with
      Hyperthreading enabled.
    id: A transient unique identifier to idenfity an instance within an
      ProvisioningConfig request.
    instanceType: Instance type. [Available
      types](https://cloud.google.com/bare-metal/docs/bms-
      planning#server_configurations)
    logicalInterfaces: List of logical interfaces for the instance. The number
      of logical interfaces will be the same as number of hardware bond/nic on
      the chosen network template. Filled if InstanceConfig.multivlan_config
      is true.
    name: Output only. The name of the instance config.
    networkConfig: The type of network configuration on the instance.
    networkTemplate: Server network template name. Filled if
      InstanceConfig.multivlan_config is true.
    osImage: OS image to initialize the instance. [Available
      images](https://cloud.google.com/bare-metal/docs/bms-
      planning#server_configurations)
    privateNetwork: Private network address, if any. Filled if
      InstanceConfig.multivlan_config is false.
    userNote: User note field, it can be used by customers to add additional
      information for the BMS Ops team .
  """

  class NetworkConfigValueValuesEnum(_messages.Enum):
    r"""The type of network configuration on the instance.

    Values:
      NETWORKCONFIG_UNSPECIFIED: The unspecified network configuration.
      SINGLE_VLAN: Instance part of single client network and single private
        network.
      MULTI_VLAN: Instance part of multiple (or single) client networks and
        private networks.
    """
    NETWORKCONFIG_UNSPECIFIED = 0
    SINGLE_VLAN = 1
    MULTI_VLAN = 2

  accountNetworksEnabled = _messages.BooleanField(1)
  clientNetwork = _messages.MessageField('NetworkAddress', 2)
  hyperthreading = _messages.BooleanField(3)
  id = _messages.StringField(4)
  instanceType = _messages.StringField(5)
  logicalInterfaces = _messages.MessageField('GoogleCloudBaremetalsolutionV2LogicalInterface', 6, repeated=True)
  name = _messages.StringField(7)
  networkConfig = _messages.EnumField('NetworkConfigValueValuesEnum', 8)
  networkTemplate = _messages.StringField(9)
  osImage = _messages.StringField(10)
  privateNetwork = _messages.MessageField('NetworkAddress', 11)
  userNote = _messages.StringField(12)


class InstanceQuota(_messages.Message):
  r"""A resource budget.

  Fields:
    availableMachineCount: Number of machines than can be created for the
      given location and instance_type.
    gcpService: The gcp service of the provisioning quota.
    instanceType: Instance type. Deprecated: use gcp_service.
    location: Location where the quota applies.
    name: Output only. The name of the instance quota.
  """

  availableMachineCount = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  gcpService = _messages.StringField(2)
  instanceType = _messages.StringField(3)
  location = _messages.StringField(4)
  name = _messages.StringField(5)


class IntakeVlanAttachment(_messages.Message):
  r"""A GCP vlan attachment.

  Fields:
    id: Identifier of the VLAN attachment.
    pairingKey: Attachment pairing key.
  """

  id = _messages.StringField(1)
  pairingKey = _messages.StringField(2)


class ListInstanceQuotasResponse(_messages.Message):
  r"""Response message for the list of Instance provisioning quotas.

  Fields:
    instanceQuotas: The provisioning quotas registered in this project.
    nextPageToken: Token to retrieve the next page of results, or empty if
      there are no more results in the list.
  """

  instanceQuotas = _messages.MessageField('InstanceQuota', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class ListInstancesResponse(_messages.Message):
  r"""Response message for the list of servers.

  Fields:
    instances: The list of servers.
    nextPageToken: A token identifying a page of results from the server.
    unreachable: Locations that could not be reached.
  """

  instances = _messages.MessageField('Instance', 1, repeated=True)
  nextPageToken = _messages.StringField(2)
  unreachable = _messages.StringField(3, repeated=True)


class ListLocationsResponse(_messages.Message):
  r"""The response message for Locations.ListLocations.

  Fields:
    locations: A list of locations that matches the specified filter in the
      request.
    nextPageToken: The standard List next-page token.
  """

  locations = _messages.MessageField('Location', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class ListLunsResponse(_messages.Message):
  r"""Response message containing the list of storage volume luns.

  Fields:
    luns: The list of luns.
    nextPageToken: A token identifying a page of results from the server.
    unreachable: Locations that could not be reached.
  """

  luns = _messages.MessageField('Lun', 1, repeated=True)
  nextPageToken = _messages.StringField(2)
  unreachable = _messages.StringField(3, repeated=True)


class ListNetworkQuotasResponse(_messages.Message):
  r"""Response message for the list of Network provisioning quotas.

  Fields:
    networkQuotas: The provisioning quotas registered in this project.
    nextPageToken: Token to retrieve the next page of results, or empty if
      there are no more results in the list.
  """

  networkQuotas = _messages.MessageField('NetworkQuota', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class ListNetworkUsageResponse(_messages.Message):
  r"""Response with Networks with IPs

  Fields:
    networks: Networks with IPs.
  """

  networks = _messages.MessageField('NetworkUsage', 1, repeated=True)


class ListNetworksResponse(_messages.Message):
  r"""Response message containing the list of networks.

  Fields:
    networks: The list of networks.
    nextPageToken: A token identifying a page of results from the server.
    unreachable: Locations that could not be reached.
  """

  networks = _messages.MessageField('Network', 1, repeated=True)
  nextPageToken = _messages.StringField(2)
  unreachable = _messages.StringField(3, repeated=True)


class ListNfsSharesResponse(_messages.Message):
  r"""Response message containing the list of NFS shares.

  Fields:
    nextPageToken: A token identifying a page of results from the server.
    nfsShares: The list of NFS shares.
    unreachable: Locations that could not be reached.
  """

  nextPageToken = _messages.StringField(1)
  nfsShares = _messages.MessageField('NfsShare', 2, repeated=True)
  unreachable = _messages.StringField(3, repeated=True)


class ListProvisioningQuotasResponse(_messages.Message):
  r"""Response message for the list of provisioning quotas.

  Fields:
    nextPageToken: Token to retrieve the next page of results, or empty if
      there are no more results in the list.
    provisioningQuotas: The provisioning quotas registered in this project.
  """

  nextPageToken = _messages.StringField(1)
  provisioningQuotas = _messages.MessageField('ProvisioningQuota', 2, repeated=True)


class ListSSHKeysResponse(_messages.Message):
  r"""Message for response of ListSSHKeys.

  Fields:
    nextPageToken: Token to retrieve the next page of results, or empty if
      there are no more results in the list.
    sshKeys: The SSH keys registered in the project.
  """

  nextPageToken = _messages.StringField(1)
  sshKeys = _messages.MessageField('SSHKey', 2, repeated=True)


class ListSnapshotSchedulePoliciesResponse(_messages.Message):
  r"""Response message containing the list of snapshot schedule policies.

  Fields:
    nextPageToken: Token to retrieve the next page of results, or empty if
      there are no more results in the list.
    snapshotSchedulePolicies: The snapshot schedule policies registered in
      this project.
  """

  nextPageToken = _messages.StringField(1)
  snapshotSchedulePolicies = _messages.MessageField('SnapshotSchedulePolicy', 2, repeated=True)


class ListStorageQuotasResponse(_messages.Message):
  r"""Response message for the list of Storage provisioning quotas.

  Fields:
    nextPageToken: Token to retrieve the next page of results, or empty if
      there are no more results in the list.
    storageQuotas: The provisioning quotas registered in this project.
  """

  nextPageToken = _messages.StringField(1)
  storageQuotas = _messages.MessageField('StorageQuota', 2, repeated=True)


class ListVolumeSnapshotsResponse(_messages.Message):
  r"""Response message containing the list of volume snapshots.

  Fields:
    nextPageToken: A token identifying a page of results from the server.
    unreachable: Locations that could not be reached.
    volumeSnapshots: The list of snapshots.
  """

  nextPageToken = _messages.StringField(1)
  unreachable = _messages.StringField(2, repeated=True)
  volumeSnapshots = _messages.MessageField('VolumeSnapshot', 3, repeated=True)


class ListVolumesResponse(_messages.Message):
  r"""Response message containing the list of storage volumes.

  Fields:
    nextPageToken: A token identifying a page of results from the server.
    unreachable: Locations that could not be reached.
    volumes: The list of storage volumes.
  """

  nextPageToken = _messages.StringField(1)
  unreachable = _messages.StringField(2, repeated=True)
  volumes = _messages.MessageField('Volume', 3, repeated=True)


class Location(_messages.Message):
  r"""A resource that represents Google Cloud Platform location.

  Messages:
    LabelsValue: Cross-service attributes for the location. For example
      {"cloud.googleapis.com/region": "us-east1"}
    MetadataValue: Service-specific metadata. For example the available
      capacity at the given location.

  Fields:
    displayName: The friendly name for this location, typically a nearby city
      name. For example, "Tokyo".
    labels: Cross-service attributes for the location. For example
      {"cloud.googleapis.com/region": "us-east1"}
    locationId: The canonical id for this location. For example: `"us-east1"`.
    metadata: Service-specific metadata. For example the available capacity at
      the given location.
    name: Resource name for the location, which may vary between
      implementations. For example: `"projects/example-project/locations/us-
      east1"`
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Cross-service attributes for the location. For example
    {"cloud.googleapis.com/region": "us-east1"}

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MetadataValue(_messages.Message):
    r"""Service-specific metadata. For example the available capacity at the
    given location.

    Messages:
      AdditionalProperty: An additional property for a MetadataValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MetadataValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  displayName = _messages.StringField(1)
  labels = _messages.MessageField('LabelsValue', 2)
  locationId = _messages.StringField(3)
  metadata = _messages.MessageField('MetadataValue', 4)
  name = _messages.StringField(5)


class LogicalNetworkInterface(_messages.Message):
  r"""Each logical network interface is effectively a network and IP pair.

  Enums:
    NetworkTypeValueValuesEnum: Type of network.

  Fields:
    defaultGateway: Whether this interface is the default gateway for the
      instance. Only one interface can be the default gateway for the
      instance.
    id: An identifier for the `Network`, generated by the backend.
    ipAddress: IP address in the network
    network: Name of the network
    networkType: Type of network.
  """

  class NetworkTypeValueValuesEnum(_messages.Enum):
    r"""Type of network.

    Values:
      TYPE_UNSPECIFIED: Unspecified value.
      CLIENT: Client network, a network peered to a Google Cloud VPC.
      PRIVATE: Private network, a network local to the Bare Metal Solution
        environment.
    """
    TYPE_UNSPECIFIED = 0
    CLIENT = 1
    PRIVATE = 2

  defaultGateway = _messages.BooleanField(1)
  id = _messages.StringField(2)
  ipAddress = _messages.StringField(3)
  network = _messages.StringField(4)
  networkType = _messages.EnumField('NetworkTypeValueValuesEnum', 5)


class Lun(_messages.Message):
  r"""A storage volume logical unit number (LUN).

  Enums:
    MultiprotocolTypeValueValuesEnum: The LUN multiprotocol type ensures the
      characteristics of the LUN are optimized for each operating system.
    StateValueValuesEnum: The state of this storage volume.
    StorageTypeValueValuesEnum: The storage type for this LUN.

  Fields:
    bootLun: Display if this LUN is a boot LUN.
    id: An identifier for the LUN, generated by the backend.
    multiprotocolType: The LUN multiprotocol type ensures the characteristics
      of the LUN are optimized for each operating system.
    name: Output only. The name of the LUN.
    shareable: Display if this LUN can be shared between multiple physical
      servers.
    sizeGb: The size of this LUN, in gigabytes.
    state: The state of this storage volume.
    storageType: The storage type for this LUN.
    storageVolume: Display the storage volume for this LUN.
    wwid: The WWID for this LUN.
  """

  class MultiprotocolTypeValueValuesEnum(_messages.Enum):
    r"""The LUN multiprotocol type ensures the characteristics of the LUN are
    optimized for each operating system.

    Values:
      MULTIPROTOCOL_TYPE_UNSPECIFIED: Server has no OS specified.
      LINUX: Server with Linux OS.
    """
    MULTIPROTOCOL_TYPE_UNSPECIFIED = 0
    LINUX = 1

  class StateValueValuesEnum(_messages.Enum):
    r"""The state of this storage volume.

    Values:
      STATE_UNSPECIFIED: The LUN is in an unknown state.
      CREATING: The LUN is being created.
      UPDATING: The LUN is being updated.
      READY: The LUN is ready for use.
      DELETING: The LUN has been requested to be deleted.
    """
    STATE_UNSPECIFIED = 0
    CREATING = 1
    UPDATING = 2
    READY = 3
    DELETING = 4

  class StorageTypeValueValuesEnum(_messages.Enum):
    r"""The storage type for this LUN.

    Values:
      STORAGE_TYPE_UNSPECIFIED: The storage type for this LUN is unknown.
      SSD: This storage type for this LUN is SSD.
      HDD: This storage type for this LUN is HDD.
    """
    STORAGE_TYPE_UNSPECIFIED = 0
    SSD = 1
    HDD = 2

  bootLun = _messages.BooleanField(1)
  id = _messages.StringField(2)
  multiprotocolType = _messages.EnumField('MultiprotocolTypeValueValuesEnum', 3)
  name = _messages.StringField(4)
  shareable = _messages.BooleanField(5)
  sizeGb = _messages.IntegerField(6)
  state = _messages.EnumField('StateValueValuesEnum', 7)
  storageType = _messages.EnumField('StorageTypeValueValuesEnum', 8)
  storageVolume = _messages.StringField(9)
  wwid = _messages.StringField(10)


class LunRange(_messages.Message):
  r"""A LUN(Logical Unit Number) range.

  Fields:
    quantity: Number of LUNs to create.
    sizeGb: The requested size of each LUN, in GB.
  """

  quantity = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  sizeGb = _messages.IntegerField(2, variant=_messages.Variant.INT32)


class Network(_messages.Message):
  r"""A Network.

  Enums:
    StateValueValuesEnum: The Network state.
    TypeValueValuesEnum: The type of this network.

  Messages:
    LabelsValue: Labels as key value pairs.

  Fields:
    cidr: The cidr of the Network.
    id: An identifier for the `Network`, generated by the backend.
    ipAddress: IP address configured.
    jumboFramesEnabled: Whether network uses standard frames or jumbo ones.
    labels: Labels as key value pairs.
    macAddress: List of physical interfaces.
    mountPoints: Input only. List of mount points to attach the network to.
    name: Output only. The resource name of this `Network`. Resource names are
      schemeless URIs that follow the conventions in
      https://cloud.google.com/apis/design/resource_names. Format:
      `projects/{project}/locations/{location}/networks/{network}`
    pod: Output only. Pod name.
    reservations: List of IP address reservations in this network. When
      updating this field, an error will be generated if a reservation
      conflicts with an IP address already allocated to a physical server.
    servicesCidr: IP range for reserved for services (e.g. NFS).
    state: The Network state.
    type: The type of this network.
    vlanId: The vlan id of the Network.
    vrf: The vrf for the Network.
  """

  class StateValueValuesEnum(_messages.Enum):
    r"""The Network state.

    Values:
      STATE_UNSPECIFIED: The Network is in an unknown state.
      PROVISIONING: The Network is provisioning.
      PROVISIONED: The Network has been provisioned.
      DEPROVISIONING: The Network is being deprovisioned.
      UPDATING: The Network is being updated.
    """
    STATE_UNSPECIFIED = 0
    PROVISIONING = 1
    PROVISIONED = 2
    DEPROVISIONING = 3
    UPDATING = 4

  class TypeValueValuesEnum(_messages.Enum):
    r"""The type of this network.

    Values:
      TYPE_UNSPECIFIED: Unspecified value.
      CLIENT: Client network, a network peered to a Google Cloud VPC.
      PRIVATE: Private network, a network local to the Bare Metal Solution
        environment.
    """
    TYPE_UNSPECIFIED = 0
    CLIENT = 1
    PRIVATE = 2

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Labels as key value pairs.

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  cidr = _messages.StringField(1)
  id = _messages.StringField(2)
  ipAddress = _messages.StringField(3)
  jumboFramesEnabled = _messages.BooleanField(4)
  labels = _messages.MessageField('LabelsValue', 5)
  macAddress = _messages.StringField(6, repeated=True)
  mountPoints = _messages.MessageField('NetworkMountPoint', 7, repeated=True)
  name = _messages.StringField(8)
  pod = _messages.StringField(9)
  reservations = _messages.MessageField('NetworkAddressReservation', 10, repeated=True)
  servicesCidr = _messages.StringField(11)
  state = _messages.EnumField('StateValueValuesEnum', 12)
  type = _messages.EnumField('TypeValueValuesEnum', 13)
  vlanId = _messages.StringField(14)
  vrf = _messages.MessageField('VRF', 15)


class NetworkAddress(_messages.Message):
  r"""A network.

  Fields:
    address: IPv4 address to be assigned to the server.
    existingNetworkId: Name of the existing network to use.
    networkId: Id of the network to use, within the same ProvisioningConfig
      request.
  """

  address = _messages.StringField(1)
  existingNetworkId = _messages.StringField(2)
  networkId = _messages.StringField(3)


class NetworkAddressReservation(_messages.Message):
  r"""A reservation of one or more addresses in a network.

  Fields:
    endAddress: The last address of this reservation block, inclusive. I.e.,
      for cases when reservations are only single addresses, end_address and
      start_address will be the same. Must be specified as a single IPv4
      address, e.g. 10.1.2.2.
    note: A note about this reservation, intended for human consumption.
    startAddress: The first address of this reservation block. Must be
      specified as a single IPv4 address, e.g. 10.1.2.2.
  """

  endAddress = _messages.StringField(1)
  note = _messages.StringField(2)
  startAddress = _messages.StringField(3)


class NetworkConfig(_messages.Message):
  r"""Configuration parameters for a new network.

  Enums:
    BandwidthValueValuesEnum: Interconnect bandwidth. Set only when type is
      CLIENT.
    ServiceCidrValueValuesEnum: Service CIDR, if any.
    TypeValueValuesEnum: The type of this network, either Client or Private.

  Fields:
    bandwidth: Interconnect bandwidth. Set only when type is CLIENT.
    cidr: CIDR range of the network.
    gcpService: The GCP service of the network. Available gcp_service are in
      https://cloud.google.com/bare-metal/docs/bms-planning.
    id: A transient unique identifier to identify a volume within an
      ProvisioningConfig request.
    jumboFramesEnabled: The JumboFramesEnabled option for customer to set.
    name: Output only. The name of the network config.
    serviceCidr: Service CIDR, if any.
    type: The type of this network, either Client or Private.
    userNote: User note field, it can be used by customers to add additional
      information for the BMS Ops team .
    vlanAttachments: List of VLAN attachments. As of now there are always 2
      attachments, but it is going to change in the future (multi vlan).
    vlanSameProject: Whether the VLAN attachment pair is located in the same
      project.
  """

  class BandwidthValueValuesEnum(_messages.Enum):
    r"""Interconnect bandwidth. Set only when type is CLIENT.

    Values:
      BANDWIDTH_UNSPECIFIED: Unspecified value.
      BW_1_GBPS: 1 Gbps.
      BW_2_GBPS: 2 Gbps.
      BW_5_GBPS: 5 Gbps.
      BW_10_GBPS: 10 Gbps.
    """
    BANDWIDTH_UNSPECIFIED = 0
    BW_1_GBPS = 1
    BW_2_GBPS = 2
    BW_5_GBPS = 3
    BW_10_GBPS = 4

  class ServiceCidrValueValuesEnum(_messages.Enum):
    r"""Service CIDR, if any.

    Values:
      SERVICE_CIDR_UNSPECIFIED: Unspecified value.
      DISABLED: Services are disabled for the given network.
      HIGH_26: Use the highest /26 block of the network to host services.
      HIGH_27: Use the highest /27 block of the network to host services.
      HIGH_28: Use the highest /28 block of the network to host services.
    """
    SERVICE_CIDR_UNSPECIFIED = 0
    DISABLED = 1
    HIGH_26 = 2
    HIGH_27 = 3
    HIGH_28 = 4

  class TypeValueValuesEnum(_messages.Enum):
    r"""The type of this network, either Client or Private.

    Values:
      TYPE_UNSPECIFIED: Unspecified value.
      CLIENT: Client network, that is a network peered to a GCP VPC.
      PRIVATE: Private network, that is a network local to the BMS POD.
    """
    TYPE_UNSPECIFIED = 0
    CLIENT = 1
    PRIVATE = 2

  bandwidth = _messages.EnumField('BandwidthValueValuesEnum', 1)
  cidr = _messages.StringField(2)
  gcpService = _messages.StringField(3)
  id = _messages.StringField(4)
  jumboFramesEnabled = _messages.BooleanField(5)
  name = _messages.StringField(6)
  serviceCidr = _messages.EnumField('ServiceCidrValueValuesEnum', 7)
  type = _messages.EnumField('TypeValueValuesEnum', 8)
  userNote = _messages.StringField(9)
  vlanAttachments = _messages.MessageField('IntakeVlanAttachment', 10, repeated=True)
  vlanSameProject = _messages.BooleanField(11)


class NetworkMountPoint(_messages.Message):
  r"""Mount point for a network.

  Fields:
    defaultGateway: Network should be a default gateway.
    instance: Instance to attach network to.
    ipAddress: Ip address of the server.
    logicalInterface: Logical interface to detach from.
  """

  defaultGateway = _messages.BooleanField(1)
  instance = _messages.StringField(2)
  ipAddress = _messages.StringField(3)
  logicalInterface = _messages.StringField(4)


class NetworkQuota(_messages.Message):
  r"""A network provisioning quota.

  Fields:
    availableBandwidthGbps: Network bandwidth, Gbps
    gcpService: The gcp service of the provisioning quota.
    name: Output only. The name of the provisioning quota.
  """

  availableBandwidthGbps = _messages.IntegerField(1)
  gcpService = _messages.StringField(2)
  name = _messages.StringField(3)


class NetworkUsage(_messages.Message):
  r"""Network with all used IP addresses.

  Fields:
    network: Network.
    usedIps: All used IP addresses in this network.
  """

  network = _messages.MessageField('Network', 1)
  usedIps = _messages.StringField(2, repeated=True)


class NfsExport(_messages.Message):
  r"""A NFS export entry.

  Enums:
    PermissionsValueValuesEnum: Export permissions.

  Fields:
    allowDev: Allow dev flag in NfsShare AllowedClientsRequest.
    allowSuid: Allow the setuid flag.
    cidr: A CIDR range.
    machineId: Either a single machine, identified by an ID, or a comma-
      separated list of machine IDs.
    networkId: Network to use to publish the export.
    noRootSquash: Disable root squashing, which is a feature of NFS. Root
      squash is a special mapping of the remote superuser (root) identity when
      using identity authentication.
    permissions: Export permissions.
  """

  class PermissionsValueValuesEnum(_messages.Enum):
    r"""Export permissions.

    Values:
      PERMISSIONS_UNSPECIFIED: Unspecified value.
      READ_ONLY: Read-only permission.
      READ_WRITE: Read-write permission.
    """
    PERMISSIONS_UNSPECIFIED = 0
    READ_ONLY = 1
    READ_WRITE = 2

  allowDev = _messages.BooleanField(1)
  allowSuid = _messages.BooleanField(2)
  cidr = _messages.StringField(3)
  machineId = _messages.StringField(4)
  networkId = _messages.StringField(5)
  noRootSquash = _messages.BooleanField(6)
  permissions = _messages.EnumField('PermissionsValueValuesEnum', 7)


class NfsShare(_messages.Message):
  r"""An NFS share.

  Enums:
    StateValueValuesEnum: Output only. The state of the NFS share.
    StorageTypeValueValuesEnum: Immutable. The storage type of the underlying
      volume.

  Messages:
    LabelsValue: Labels as key value pairs.

  Fields:
    allowedClients: List of allowed access points.
    id: Output only. An identifier for the NFS share, generated by the
      backend. This is the same value as nfs_share_id and will replace it in
      the future.
    labels: Labels as key value pairs.
    name: Immutable. The name of the NFS share.
    nfsShareId: Output only. An identifier for the NFS share, generated by the
      backend. This field will be deprecated in the future, use `id` instead.
    requestedSizeGib: The requested size, in GiB.
    state: Output only. The state of the NFS share.
    storageType: Immutable. The storage type of the underlying volume.
    volume: Output only. The underlying volume of the share. Created
      automatically during provisioning.
  """

  class StateValueValuesEnum(_messages.Enum):
    r"""Output only. The state of the NFS share.

    Values:
      STATE_UNSPECIFIED: The share is in an unknown state.
      PROVISIONED: The share has been provisioned.
      CREATING: The NFS Share is being created.
      UPDATING: The NFS Share is being updated.
      DELETING: The NFS Share has been requested to be deleted.
    """
    STATE_UNSPECIFIED = 0
    PROVISIONED = 1
    CREATING = 2
    UPDATING = 3
    DELETING = 4

  class StorageTypeValueValuesEnum(_messages.Enum):
    r"""Immutable. The storage type of the underlying volume.

    Values:
      STORAGE_TYPE_UNSPECIFIED: The storage type for this volume is unknown.
      SSD: The storage type for this volume is SSD.
      HDD: This storage type for this volume is HDD.
    """
    STORAGE_TYPE_UNSPECIFIED = 0
    SSD = 1
    HDD = 2

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Labels as key value pairs.

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  allowedClients = _messages.MessageField('AllowedClient', 1, repeated=True)
  id = _messages.StringField(2)
  labels = _messages.MessageField('LabelsValue', 3)
  name = _messages.StringField(4)
  nfsShareId = _messages.StringField(5)
  requestedSizeGib = _messages.IntegerField(6)
  state = _messages.EnumField('StateValueValuesEnum', 7)
  storageType = _messages.EnumField('StorageTypeValueValuesEnum', 8)
  volume = _messages.StringField(9)


class OSImage(_messages.Message):
  r"""Operation System image.

  Fields:
    applicableInstanceTypes: Instance types this image is applicable to.
      [Available types](https://cloud.google.com/bare-metal/docs/bms-
      planning#server_configurations)
    code: OS Image code.
    description: OS Image description.
    name: Output only. OS Image's unique name.
    supportedNetworkTemplates: Network templates that can be used with this OS
      Image.
  """

  applicableInstanceTypes = _messages.StringField(1, repeated=True)
  code = _messages.StringField(2)
  description = _messages.StringField(3)
  name = _messages.StringField(4)
  supportedNetworkTemplates = _messages.MessageField('ServerNetworkTemplate', 5, repeated=True)


class Operation(_messages.Message):
  r"""This resource represents a long-running operation that is the result of
  a network API call.

  Messages:
    MetadataValue: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    ResponseValue: The normal response of the operation in case of success. If
      the original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

  Fields:
    done: If the value is `false`, it means the operation is still in
      progress. If `true`, the operation is completed, and either `error` or
      `response` is available.
    error: The error result of the operation in case of failure or
      cancellation.
    metadata: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    name: The server-assigned name, which is only unique within the same
      service that originally returns it. If you use the default HTTP mapping,
      the `name` should be a resource name ending with
      `operations/{unique_id}`.
    response: The normal response of the operation in case of success. If the
      original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MetadataValue(_messages.Message):
    r"""Service-specific metadata associated with the operation. It typically
    contains progress information and common metadata such as create time.
    Some services might not provide such metadata. Any method that returns a
    long-running operation should document the metadata type, if any.

    Messages:
      AdditionalProperty: An additional property for a MetadataValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MetadataValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class ResponseValue(_messages.Message):
    r"""The normal response of the operation in case of success. If the
    original method returns no data on success, such as `Delete`, the response
    is `google.protobuf.Empty`. If the original method is standard
    `Get`/`Create`/`Update`, the response should be the resource. For other
    methods, the response should have the type `XxxResponse`, where `Xxx` is
    the original method name. For example, if the original method name is
    `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

    Messages:
      AdditionalProperty: An additional property for a ResponseValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a ResponseValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  done = _messages.BooleanField(1)
  error = _messages.MessageField('Status', 2)
  metadata = _messages.MessageField('MetadataValue', 3)
  name = _messages.StringField(4)
  response = _messages.MessageField('ResponseValue', 5)


class OperationMetadata(_messages.Message):
  r"""Represents the metadata from a long-running operation.

  Fields:
    apiVersion: Output only. API version used with the operation.
    createTime: Output only. The time the operation was created.
    endTime: Output only. The time the operation finished running.
    requestedCancellation: Output only. Identifies whether the user requested
      the cancellation of the operation. Operations that have been
      successfully cancelled have Operation.error value with a
      google.rpc.Status.code of 1, corresponding to `Code.CANCELLED`.
    statusMessage: Output only. Human-readable status of the operation, if
      any.
    target: Output only. Server-defined resource path for the target of the
      operation.
    verb: Output only. Name of the action executed by the operation.
  """

  apiVersion = _messages.StringField(1)
  createTime = _messages.StringField(2)
  endTime = _messages.StringField(3)
  requestedCancellation = _messages.BooleanField(4)
  statusMessage = _messages.StringField(5)
  target = _messages.StringField(6)
  verb = _messages.StringField(7)


class ProvisioningConfig(_messages.Message):
  r"""A provisioning configuration.

  Enums:
    StateValueValuesEnum: Output only. State of ProvisioningConfig.

  Fields:
    cloudConsoleUri: Output only. URI to Cloud Console UI view of this
      provisioning config.
    customId: Optional. The user-defined identifier of the provisioning
      config.
    email: Email provided to send a confirmation with provisioning config to.
      Deprecated in favour of email field in request messages.
    handoverServiceAccount: A service account to enable customers to access
      instance credentials upon handover.
    instances: Instances to be created.
    location: Optional. Location name of this ProvisioningConfig. It is
      optional only for Intake UI transition period.
    name: Output only. The system-generated name of the provisioning config.
      This follows the UUID format.
    networks: Networks to be created.
    state: Output only. State of ProvisioningConfig.
    statusMessage: Optional status messages associated with the FAILED state.
    ticketId: A generated ticket id to track provisioning request.
    updateTime: Output only. Last update timestamp.
    volumes: Volumes to be created.
    vpcScEnabled: If true, VPC SC is enabled for the cluster.
  """

  class StateValueValuesEnum(_messages.Enum):
    r"""Output only. State of ProvisioningConfig.

    Values:
      STATE_UNSPECIFIED: State wasn't specified.
      DRAFT: ProvisioningConfig is a draft and can be freely modified.
      SUBMITTED: ProvisioningConfig was already submitted and cannot be
        modified.
      PROVISIONING: ProvisioningConfig was in the provisioning state.
        Initially this state comes from the work order table in big query when
        SNOW is used. Later this field can be set by the work order API.
      PROVISIONED: ProvisioningConfig was provisioned, meaning the resources
        exist.
      VALIDATED: ProvisioningConfig was validated. A validation tool will be
        run to set this state.
      CANCELLED: ProvisioningConfig was canceled.
      FAILED: The request is submitted for provisioning, with error return.
    """
    STATE_UNSPECIFIED = 0
    DRAFT = 1
    SUBMITTED = 2
    PROVISIONING = 3
    PROVISIONED = 4
    VALIDATED = 5
    CANCELLED = 6
    FAILED = 7

  cloudConsoleUri = _messages.StringField(1)
  customId = _messages.StringField(2)
  email = _messages.StringField(3)
  handoverServiceAccount = _messages.StringField(4)
  instances = _messages.MessageField('InstanceConfig', 5, repeated=True)
  location = _messages.StringField(6)
  name = _messages.StringField(7)
  networks = _messages.MessageField('NetworkConfig', 8, repeated=True)
  state = _messages.EnumField('StateValueValuesEnum', 9)
  statusMessage = _messages.StringField(10)
  ticketId = _messages.StringField(11)
  updateTime = _messages.StringField(12)
  volumes = _messages.MessageField('VolumeConfig', 13, repeated=True)
  vpcScEnabled = _messages.BooleanField(14)


class ProvisioningQuota(_messages.Message):
  r"""A provisioning quota for a given project.

  Enums:
    AssetTypeValueValuesEnum: The asset type of this provisioning quota.

  Fields:
    assetType: The asset type of this provisioning quota.
    availableCount: The available count of the provisioning quota.
    gcpService: The gcp service of the provisioning quota.
    instanceQuota: Instance quota.
    location: The specific location of the provisioining quota.
    name: Output only. The name of the provisioning quota.
    networkBandwidth: Network bandwidth, Gbps
    serverCount: Server count.
    storageGib: Storage size (GB).
  """

  class AssetTypeValueValuesEnum(_messages.Enum):
    r"""The asset type of this provisioning quota.

    Values:
      ASSET_TYPE_UNSPECIFIED: The unspecified type.
      ASSET_TYPE_SERVER: The server asset type.
      ASSET_TYPE_STORAGE: The storage asset type.
      ASSET_TYPE_NETWORK: The network asset type.
    """
    ASSET_TYPE_UNSPECIFIED = 0
    ASSET_TYPE_SERVER = 1
    ASSET_TYPE_STORAGE = 2
    ASSET_TYPE_NETWORK = 3

  assetType = _messages.EnumField('AssetTypeValueValuesEnum', 1)
  availableCount = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  gcpService = _messages.StringField(3)
  instanceQuota = _messages.MessageField('InstanceQuota', 4)
  location = _messages.StringField(5)
  name = _messages.StringField(6)
  networkBandwidth = _messages.IntegerField(7)
  serverCount = _messages.IntegerField(8)
  storageGib = _messages.IntegerField(9)


class QosPolicy(_messages.Message):
  r"""QOS policy parameters.

  Fields:
    bandwidthGbps: The bandwidth permitted by the QOS policy, in gbps.
  """

  bandwidthGbps = _messages.FloatField(1)


class ResetInstanceRequest(_messages.Message):
  r"""Message requesting to reset a server."""


class ResizeVolumeRequest(_messages.Message):
  r"""Request for emergency resize Volume.

  Fields:
    sizeGib: New Volume size, in GiB.
  """

  sizeGib = _messages.IntegerField(1)


class RestoreVolumeSnapshotRequest(_messages.Message):
  r"""Message for restoring a volume snapshot."""


class SSHKey(_messages.Message):
  r"""An SSH key, used for authorizing with the interactive serial console
  feature.

  Fields:
    name: Output only. The name of this SSH key. Currently, the only valid
      value for the location is "global".
    publicKey: The public SSH key. This must be in OpenSSH .authorized_keys
      format.
  """

  name = _messages.StringField(1)
  publicKey = _messages.StringField(2)


class Schedule(_messages.Message):
  r"""A snapshot schedule.

  Fields:
    crontabSpec: A crontab-like specification that the schedule uses to take
      snapshots.
    prefix: A list of snapshot names created in this schedule.
    retentionCount: The maximum number of snapshots to retain in this
      schedule.
  """

  crontabSpec = _messages.StringField(1)
  prefix = _messages.StringField(2)
  retentionCount = _messages.IntegerField(3, variant=_messages.Variant.INT32)


class ServerNetworkTemplate(_messages.Message):
  r"""Network template.

  Fields:
    applicableInstanceTypes: Instance types this template is applicable to.
    logicalInterfaces: Logical interfaces.
    name: Output only. Template's unique name. The full resource name follows
      the pattern: `projects/{project}/locations/{location}/serverNetworkTempl
      ate/{server_network_template}` Generally, the {server_network_template}
      follows the syntax of "bond" or "nic".
  """

  applicableInstanceTypes = _messages.StringField(1, repeated=True)
  logicalInterfaces = _messages.MessageField('GoogleCloudBaremetalsolutionV2ServerNetworkTemplateLogicalInterface', 2, repeated=True)
  name = _messages.StringField(3)


class SnapshotReservationDetail(_messages.Message):
  r"""Details about snapshot space reservation and usage on the storage
  volume.

  Fields:
    reservedSpaceGib: The space on this storage volume reserved for snapshots,
      shown in GiB.
    reservedSpacePercent: Percent of the total Volume size reserved for
      snapshot copies. Enabling snapshots requires reserving 20% or more of
      the storage volume space for snapshots. Maximum reserved space for
      snapshots is 40%. Setting this field will effectively set
      snapshot_enabled to true.
    reservedSpaceRemainingGib: The amount, in GiB, of available space in this
      storage volume's reserved snapshot space.
    reservedSpaceUsedPercent: The percent of snapshot space on this storage
      volume actually being used by the snapshot copies. This value might be
      higher than 100% if the snapshot copies have overflowed into the data
      portion of the storage volume.
  """

  reservedSpaceGib = _messages.IntegerField(1)
  reservedSpacePercent = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  reservedSpaceRemainingGib = _messages.IntegerField(3)
  reservedSpaceUsedPercent = _messages.IntegerField(4, variant=_messages.Variant.INT32)


class SnapshotSchedulePolicy(_messages.Message):
  r"""A snapshot schedule policy.

  Enums:
    StateValueValuesEnum: The state of the snapshot schedule policy.

  Messages:
    LabelsValue: Labels as key value pairs.

  Fields:
    description: The description of the snapshot schedule policy.
    id: An identifier for the snapshot schedule policy, generated by the
      backend.
    labels: Labels as key value pairs.
    name: Output only. The name of the snapshot schedule policy.
    schedules: The snapshot schedules contained in this policy. You can
      specify a maximum of 5 schedules.
    state: The state of the snapshot schedule policy.
  """

  class StateValueValuesEnum(_messages.Enum):
    r"""The state of the snapshot schedule policy.

    Values:
      STATE_UNSPECIFIED: The policy is in an unknown state.
      PROVISIONED: The policy is been provisioned.
    """
    STATE_UNSPECIFIED = 0
    PROVISIONED = 1

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Labels as key value pairs.

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  description = _messages.StringField(1)
  id = _messages.StringField(2)
  labels = _messages.MessageField('LabelsValue', 3)
  name = _messages.StringField(4)
  schedules = _messages.MessageField('Schedule', 5, repeated=True)
  state = _messages.EnumField('StateValueValuesEnum', 6)


class StandardQueryParameters(_messages.Message):
  r"""Query parameters accepted by all methods.

  Enums:
    FXgafvValueValuesEnum: V1 error format.
    AltValueValuesEnum: Data format for response.

  Fields:
    f__xgafv: V1 error format.
    access_token: OAuth access token.
    alt: Data format for response.
    callback: JSONP
    fields: Selector specifying which fields to include in a partial response.
    key: API key. Your API key identifies your project and provides you with
      API access, quota, and reports. Required unless you provide an OAuth 2.0
      token.
    oauth_token: OAuth 2.0 token for the current user.
    prettyPrint: Returns response with indentations and line breaks.
    quotaUser: Available to use for quota purposes for server-side
      applications. Can be any arbitrary string assigned to a user, but should
      not exceed 40 characters.
    trace: A tracing token of the form "token:<tokenid>" to include in api
      requests.
    uploadType: Legacy upload protocol for media (e.g. "media", "multipart").
    upload_protocol: Upload protocol for media (e.g. "raw", "multipart").
  """

  class AltValueValuesEnum(_messages.Enum):
    r"""Data format for response.

    Values:
      json: Responses with Content-Type of application/json
      media: Media download with context-dependent Content-Type
      proto: Responses with Content-Type of application/x-protobuf
    """
    json = 0
    media = 1
    proto = 2

  class FXgafvValueValuesEnum(_messages.Enum):
    r"""V1 error format.

    Values:
      _1: v1 error format
      _2: v2 error format
    """
    _1 = 0
    _2 = 1

  f__xgafv = _messages.EnumField('FXgafvValueValuesEnum', 1)
  access_token = _messages.StringField(2)
  alt = _messages.EnumField('AltValueValuesEnum', 3, default='json')
  callback = _messages.StringField(4)
  fields = _messages.StringField(5)
  key = _messages.StringField(6)
  oauth_token = _messages.StringField(7)
  prettyPrint = _messages.BooleanField(8, default=True)
  quotaUser = _messages.StringField(9)
  trace = _messages.StringField(10)
  uploadType = _messages.StringField(11)
  upload_protocol = _messages.StringField(12)


class StartInstanceRequest(_messages.Message):
  r"""Message requesting to start a server."""


class Status(_messages.Message):
  r"""The `Status` type defines a logical error model that is suitable for
  different programming environments, including REST APIs and RPC APIs. It is
  used by [gRPC](https://github.com/grpc). Each `Status` message contains
  three pieces of data: error code, error message, and error details. You can
  find out more about this error model and how to work with it in the [API
  Design Guide](https://cloud.google.com/apis/design/errors).

  Messages:
    DetailsValueListEntry: A DetailsValueListEntry object.

  Fields:
    code: The status code, which should be an enum value of google.rpc.Code.
    details: A list of messages that carry the error details. There is a
      common set of message types for APIs to use.
    message: A developer-facing error message, which should be in English. Any
      user-facing error message should be localized and sent in the
      google.rpc.Status.details field, or localized by the client.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class DetailsValueListEntry(_messages.Message):
    r"""A DetailsValueListEntry object.

    Messages:
      AdditionalProperty: An additional property for a DetailsValueListEntry
        object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a DetailsValueListEntry object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  code = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  details = _messages.MessageField('DetailsValueListEntry', 2, repeated=True)
  message = _messages.StringField(3)


class StopInstanceRequest(_messages.Message):
  r"""Message requesting to stop a server."""


class StorageQuota(_messages.Message):
  r"""A storage provisioning quota .

  Fields:
    availableGib: Storage size (GiB).
    gcpService: The gcp service of the provisioning quota.
    name: Output only. The name of the provisioning quota.
  """

  availableGib = _messages.IntegerField(1)
  gcpService = _messages.StringField(2)
  name = _messages.StringField(3)


class SubmitProvisioningConfigRequest(_messages.Message):
  r"""Request for SubmitProvisioningConfig.

  Fields:
    email: Optional. Email provided to send a confirmation with provisioning
      config to.
    provisioningConfig: Required. The ProvisioningConfig to create.
  """

  email = _messages.StringField(1)
  provisioningConfig = _messages.MessageField('ProvisioningConfig', 2)


class SubmitProvisioningConfigResponse(_messages.Message):
  r"""Response for SubmitProvisioningConfig.

  Fields:
    provisioningConfig: The submitted provisioning config.
  """

  provisioningConfig = _messages.MessageField('ProvisioningConfig', 1)


class VRF(_messages.Message):
  r"""A network VRF.

  Enums:
    StateValueValuesEnum: The possible state of VRF.

  Fields:
    name: The name of the VRF.
    qosPolicy: The QOS policy applied to this VRF. The value is only
      meaningful when all the vlan attachments have the same QoS. This field
      should not be used for new integrations, use vlan attachment level qos
      instead. The field is left for backward-compatibility.
    state: The possible state of VRF.
    vlanAttachments: The list of VLAN attachments for the VRF.
  """

  class StateValueValuesEnum(_messages.Enum):
    r"""The possible state of VRF.

    Values:
      STATE_UNSPECIFIED: The unspecified state.
      PROVISIONING: The vrf is provisioning.
      PROVISIONED: The vrf is provisioned.
    """
    STATE_UNSPECIFIED = 0
    PROVISIONING = 1
    PROVISIONED = 2

  name = _messages.StringField(1)
  qosPolicy = _messages.MessageField('QosPolicy', 2)
  state = _messages.EnumField('StateValueValuesEnum', 3)
  vlanAttachments = _messages.MessageField('VlanAttachment', 4, repeated=True)


class VlanAttachment(_messages.Message):
  r"""VLAN attachment details.

  Fields:
    id: Immutable. The identifier of the attachment within vrf.
    pairingKey: Input only. Pairing key.
    peerIp: The peer IP of the attachment.
    peerVlanId: The peer vlan ID of the attachment.
    qosPolicy: The QOS policy applied to this VLAN attachment. This value
      should be preferred to using qos at vrf level.
    routerIp: The router IP of the attachment.
  """

  id = _messages.StringField(1)
  pairingKey = _messages.StringField(2)
  peerIp = _messages.StringField(3)
  peerVlanId = _messages.IntegerField(4)
  qosPolicy = _messages.MessageField('QosPolicy', 5)
  routerIp = _messages.StringField(6)


class Volume(_messages.Message):
  r"""A storage volume.

  Enums:
    PerformanceTierValueValuesEnum: Immutable. Performance tier of the Volume.
      Default is SHARED.
    ProtocolValueValuesEnum: Output only. Storage protocol for the Volume.
    SnapshotAutoDeleteBehaviorValueValuesEnum: The behavior to use when
      snapshot reserved space is full.
    StateValueValuesEnum: The state of this storage volume.
    StorageTypeValueValuesEnum: The storage type for this volume.

  Messages:
    LabelsValue: Labels as key value pairs.

  Fields:
    autoGrownSizeGib: The size, in GiB, that this storage volume has expanded
      as a result of an auto grow policy. In the absence of auto-grow, the
      value is 0.
    bootVolume: Output only. Whether this volume is a boot volume. A boot
      volume is one which contains a boot LUN.
    currentSizeGib: The current size of this storage volume, in GiB, including
      space reserved for snapshots. This size might be different than the
      requested size if the storage volume has been configured with auto grow
      or auto shrink.
    emergencySizeGib: Additional emergency size that was requested for this
      Volume, in GiB. current_size_gib includes this value.
    id: An identifier for the `Volume`, generated by the backend.
    labels: Labels as key value pairs.
    maxSizeGib: Maximum size volume can be expanded to in case of evergency,
      in GiB.
    name: Output only. The resource name of this `Volume`. Resource names are
      schemeless URIs that follow the conventions in
      https://cloud.google.com/apis/design/resource_names. Format:
      `projects/{project}/locations/{location}/volumes/{volume}`
    notes: Input only. User-specified notes for new Volume. Used to provision
      Volumes that require manual intervention.
    originallyRequestedSizeGib: Originally requested size, in GiB.
    performanceTier: Immutable. Performance tier of the Volume. Default is
      SHARED.
    pod: Immutable. Pod name.
    protocol: Output only. Storage protocol for the Volume.
    remainingSpaceGib: The space remaining in the storage volume for new LUNs,
      in GiB, excluding space reserved for snapshots.
    requestedSizeGib: The requested size of this storage volume, in GiB.
    snapshotAutoDeleteBehavior: The behavior to use when snapshot reserved
      space is full.
    snapshotEnabled: Whether snapshots are enabled.
    snapshotReservationDetail: Details about snapshot space reservation and
      usage on the storage volume.
    snapshotSchedulePolicy: The name of the snapshot schedule policy in use
      for this volume, if any.
    state: The state of this storage volume.
    storageType: The storage type for this volume.
  """

  class PerformanceTierValueValuesEnum(_messages.Enum):
    r"""Immutable. Performance tier of the Volume. Default is SHARED.

    Values:
      VOLUME_PERFORMANCE_TIER_UNSPECIFIED: Value is not specified.
      VOLUME_PERFORMANCE_TIER_SHARED: Regular volumes, shared aggregates.
      VOLUME_PERFORMANCE_TIER_ASSIGNED: Assigned aggregates.
      VOLUME_PERFORMANCE_TIER_HT: High throughput aggregates.
    """
    VOLUME_PERFORMANCE_TIER_UNSPECIFIED = 0
    VOLUME_PERFORMANCE_TIER_SHARED = 1
    VOLUME_PERFORMANCE_TIER_ASSIGNED = 2
    VOLUME_PERFORMANCE_TIER_HT = 3

  class ProtocolValueValuesEnum(_messages.Enum):
    r"""Output only. Storage protocol for the Volume.

    Values:
      PROTOCOL_UNSPECIFIED: Value is not specified.
      FIBRE_CHANNEL: Fibre Channel protocol.
      NFS: NFS protocol means Volume is a NFS Share volume. Such volumes
        cannot be manipulated via Volumes API.
    """
    PROTOCOL_UNSPECIFIED = 0
    FIBRE_CHANNEL = 1
    NFS = 2

  class SnapshotAutoDeleteBehaviorValueValuesEnum(_messages.Enum):
    r"""The behavior to use when snapshot reserved space is full.

    Values:
      SNAPSHOT_AUTO_DELETE_BEHAVIOR_UNSPECIFIED: The unspecified behavior.
      DISABLED: Don't delete any snapshots. This disables new snapshot
        creation, as long as the snapshot reserved space is full.
      OLDEST_FIRST: Delete the oldest snapshots first.
      NEWEST_FIRST: Delete the newest snapshots first.
    """
    SNAPSHOT_AUTO_DELETE_BEHAVIOR_UNSPECIFIED = 0
    DISABLED = 1
    OLDEST_FIRST = 2
    NEWEST_FIRST = 3

  class StateValueValuesEnum(_messages.Enum):
    r"""The state of this storage volume.

    Values:
      STATE_UNSPECIFIED: The storage volume is in an unknown state.
      CREATING: The storage volume is being created.
      READY: The storage volume is ready for use.
      DELETING: The storage volume has been requested to be deleted.
      UPDATING: The storage volume is being updated.
    """
    STATE_UNSPECIFIED = 0
    CREATING = 1
    READY = 2
    DELETING = 3
    UPDATING = 4

  class StorageTypeValueValuesEnum(_messages.Enum):
    r"""The storage type for this volume.

    Values:
      STORAGE_TYPE_UNSPECIFIED: The storage type for this volume is unknown.
      SSD: The storage type for this volume is SSD.
      HDD: This storage type for this volume is HDD.
    """
    STORAGE_TYPE_UNSPECIFIED = 0
    SSD = 1
    HDD = 2

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Labels as key value pairs.

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  autoGrownSizeGib = _messages.IntegerField(1)
  bootVolume = _messages.BooleanField(2)
  currentSizeGib = _messages.IntegerField(3)
  emergencySizeGib = _messages.IntegerField(4)
  id = _messages.StringField(5)
  labels = _messages.MessageField('LabelsValue', 6)
  maxSizeGib = _messages.IntegerField(7)
  name = _messages.StringField(8)
  notes = _messages.StringField(9)
  originallyRequestedSizeGib = _messages.IntegerField(10)
  performanceTier = _messages.EnumField('PerformanceTierValueValuesEnum', 11)
  pod = _messages.StringField(12)
  protocol = _messages.EnumField('ProtocolValueValuesEnum', 13)
  remainingSpaceGib = _messages.IntegerField(14)
  requestedSizeGib = _messages.IntegerField(15)
  snapshotAutoDeleteBehavior = _messages.EnumField('SnapshotAutoDeleteBehaviorValueValuesEnum', 16)
  snapshotEnabled = _messages.BooleanField(17)
  snapshotReservationDetail = _messages.MessageField('SnapshotReservationDetail', 18)
  snapshotSchedulePolicy = _messages.StringField(19)
  state = _messages.EnumField('StateValueValuesEnum', 20)
  storageType = _messages.EnumField('StorageTypeValueValuesEnum', 21)


class VolumeConfig(_messages.Message):
  r"""Configuration parameters for a new volume.

  Enums:
    PerformanceTierValueValuesEnum: Performance tier of the Volume. Default is
      SHARED.
    ProtocolValueValuesEnum: Volume protocol.
    TypeValueValuesEnum: The type of this Volume.

  Fields:
    gcpService: The GCP service of the storage volume. Available gcp_service
      are in https://cloud.google.com/bare-metal/docs/bms-planning.
    id: A transient unique identifier to identify a volume within an
      ProvisioningConfig request.
    lunRanges: LUN ranges to be configured. Set only when protocol is
      PROTOCOL_FC.
    machineIds: Machine ids connected to this volume. Set only when protocol
      is PROTOCOL_FC.
    name: Output only. The name of the volume config.
    nfsExports: NFS exports. Set only when protocol is PROTOCOL_NFS.
    performanceTier: Performance tier of the Volume. Default is SHARED.
    protocol: Volume protocol.
    sizeGb: The requested size of this volume, in GB.
    snapshotsEnabled: Whether snapshots should be enabled.
    type: The type of this Volume.
    userNote: User note field, it can be used by customers to add additional
      information for the BMS Ops team .
  """

  class PerformanceTierValueValuesEnum(_messages.Enum):
    r"""Performance tier of the Volume. Default is SHARED.

    Values:
      VOLUME_PERFORMANCE_TIER_UNSPECIFIED: Value is not specified.
      VOLUME_PERFORMANCE_TIER_SHARED: Regular volumes, shared aggregates.
      VOLUME_PERFORMANCE_TIER_ASSIGNED: Assigned aggregates.
      VOLUME_PERFORMANCE_TIER_HT: High throughput aggregates.
    """
    VOLUME_PERFORMANCE_TIER_UNSPECIFIED = 0
    VOLUME_PERFORMANCE_TIER_SHARED = 1
    VOLUME_PERFORMANCE_TIER_ASSIGNED = 2
    VOLUME_PERFORMANCE_TIER_HT = 3

  class ProtocolValueValuesEnum(_messages.Enum):
    r"""Volume protocol.

    Values:
      PROTOCOL_UNSPECIFIED: Unspecified value.
      PROTOCOL_FC: Fibre channel.
      PROTOCOL_NFS: Network file system.
    """
    PROTOCOL_UNSPECIFIED = 0
    PROTOCOL_FC = 1
    PROTOCOL_NFS = 2

  class TypeValueValuesEnum(_messages.Enum):
    r"""The type of this Volume.

    Values:
      TYPE_UNSPECIFIED: The unspecified type.
      FLASH: This Volume is on flash.
      DISK: This Volume is on disk.
    """
    TYPE_UNSPECIFIED = 0
    FLASH = 1
    DISK = 2

  gcpService = _messages.StringField(1)
  id = _messages.StringField(2)
  lunRanges = _messages.MessageField('LunRange', 3, repeated=True)
  machineIds = _messages.StringField(4, repeated=True)
  name = _messages.StringField(5)
  nfsExports = _messages.MessageField('NfsExport', 6, repeated=True)
  performanceTier = _messages.EnumField('PerformanceTierValueValuesEnum', 7)
  protocol = _messages.EnumField('ProtocolValueValuesEnum', 8)
  sizeGb = _messages.IntegerField(9, variant=_messages.Variant.INT32)
  snapshotsEnabled = _messages.BooleanField(10)
  type = _messages.EnumField('TypeValueValuesEnum', 11)
  userNote = _messages.StringField(12)


class VolumeLunRange(_messages.Message):
  r"""A LUN(Logical Unit Number) range.

  Fields:
    quantity: Number of LUNs to create.
    sizeGb: The requested size of each LUN, in GB.
  """

  quantity = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  sizeGb = _messages.IntegerField(2, variant=_messages.Variant.INT32)


class VolumeSnapshot(_messages.Message):
  r"""A snapshot of a volume. Only boot volumes can have snapshots.

  Enums:
    TypeValueValuesEnum: Output only. The type of the snapshot which indicates
      whether it was scheduled or manual/ad-hoc.

  Fields:
    createTime: Output only. The creation time of the snapshot.
    description: The description of the snapshot.
    id: Output only. An identifier for the snapshot, generated by the backend.
    name: The name of the snapshot.
    storageVolume: Output only. The name of the volume which this snapshot
      belongs to.
    type: Output only. The type of the snapshot which indicates whether it was
      scheduled or manual/ad-hoc.
  """

  class TypeValueValuesEnum(_messages.Enum):
    r"""Output only. The type of the snapshot which indicates whether it was
    scheduled or manual/ad-hoc.

    Values:
      SNAPSHOT_TYPE_UNSPECIFIED: Type is not specified.
      AD_HOC: Snapshot was taken manually by user.
      SCHEDULED: Snapshot was taken automatically as a part of a snapshot
        schedule.
    """
    SNAPSHOT_TYPE_UNSPECIFIED = 0
    AD_HOC = 1
    SCHEDULED = 2

  createTime = _messages.StringField(1)
  description = _messages.StringField(2)
  id = _messages.StringField(3)
  name = _messages.StringField(4)
  storageVolume = _messages.StringField(5)
  type = _messages.EnumField('TypeValueValuesEnum', 6)


encoding.AddCustomJsonFieldMapping(
    StandardQueryParameters, 'f__xgafv', '$.xgafv')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_1', '1')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_2', '2')



"""Generated client library for cloudscheduler version v1alpha1."""
# NOTE: This file is autogenerated and should not be edited by hand.

from __future__ import absolute_import

from apitools.base.py import base_api
from googlecloudsdk.third_party.apis.cloudscheduler.v1alpha1 import cloudscheduler_v1alpha1_messages as messages


class CloudschedulerV1alpha1(base_api.BaseApiClient):
  """Generated client library for service cloudscheduler version v1alpha1."""

  MESSAGES_MODULE = messages
  BASE_URL = 'https://cloudscheduler.googleapis.com/'
  MTLS_BASE_URL = 'https://cloudscheduler.mtls.googleapis.com/'

  _PACKAGE = 'cloudscheduler'
  _SCOPES = ['https://www.googleapis.com/auth/cloud-platform']
  _VERSION = 'v1alpha1'
  _CLIENT_ID = 'CLIENT_ID'
  _CLIENT_SECRET = 'CLIENT_SECRET'
  _USER_AGENT = 'google-cloud-sdk'
  _CLIENT_CLASS_NAME = 'CloudschedulerV1alpha1'
  _URL_VERSION = 'v1alpha1'
  _API_KEY = None

  def __init__(self, url='', credentials=None,
               get_credentials=True, http=None, model=None,
               log_request=False, log_response=False,
               credentials_args=None, default_global_params=None,
               additional_http_headers=None, response_encoding=None):
    """Create a new cloudscheduler handle."""
    url = url or self.BASE_URL
    super(CloudschedulerV1alpha1, self).__init__(
        url, credentials=credentials,
        get_credentials=get_credentials, http=http, model=model,
        log_request=log_request, log_response=log_response,
        credentials_args=credentials_args,
        default_global_params=default_global_params,
        additional_http_headers=additional_http_headers,
        response_encoding=response_encoding)
    self.projects_locations_jobs = self.ProjectsLocationsJobsService(self)
    self.projects_locations = self.ProjectsLocationsService(self)
    self.projects = self.ProjectsService(self)

  class ProjectsLocationsJobsService(base_api.BaseApiService):
    """Service class for the projects_locations_jobs resource."""

    _NAME = 'projects_locations_jobs'

    def __init__(self, client):
      super(CloudschedulerV1alpha1.ProjectsLocationsJobsService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a job.

      Args:
        request: (CloudschedulerProjectsLocationsJobsCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Job) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/projects/{projectsId}/locations/{locationsId}/jobs',
        http_method='POST',
        method_id='cloudscheduler.projects.locations.jobs.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=[],
        relative_path='v1alpha1/{+parent}/jobs',
        request_field='job',
        request_type_name='CloudschedulerProjectsLocationsJobsCreateRequest',
        response_type_name='Job',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a job.

      Args:
        request: (CloudschedulerProjectsLocationsJobsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/projects/{projectsId}/locations/{locationsId}/jobs/{jobsId}',
        http_method='DELETE',
        method_id='cloudscheduler.projects.locations.jobs.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['legacyAppEngineCron'],
        relative_path='v1alpha1/{+name}',
        request_field='',
        request_type_name='CloudschedulerProjectsLocationsJobsDeleteRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets a job.

      Args:
        request: (CloudschedulerProjectsLocationsJobsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Job) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/projects/{projectsId}/locations/{locationsId}/jobs/{jobsId}',
        http_method='GET',
        method_id='cloudscheduler.projects.locations.jobs.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha1/{+name}',
        request_field='',
        request_type_name='CloudschedulerProjectsLocationsJobsGetRequest',
        response_type_name='Job',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists jobs.

      Args:
        request: (CloudschedulerProjectsLocationsJobsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListJobsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/projects/{projectsId}/locations/{locationsId}/jobs',
        http_method='GET',
        method_id='cloudscheduler.projects.locations.jobs.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'legacyAppEngineCron', 'pageSize', 'pageToken'],
        relative_path='v1alpha1/{+parent}/jobs',
        request_field='',
        request_type_name='CloudschedulerProjectsLocationsJobsListRequest',
        response_type_name='ListJobsResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates a job. If successful, the updated Job is returned. If the job does not exist, `NOT_FOUND` is returned. If UpdateJob does not successfully return, it is possible for the job to be in an Job.State.UPDATE_FAILED state. A job in this state may not be executed. If this happens, retry the UpdateJob request until a successful response is received.

      Args:
        request: (CloudschedulerProjectsLocationsJobsPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Job) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/projects/{projectsId}/locations/{locationsId}/jobs/{jobsId}',
        http_method='PATCH',
        method_id='cloudscheduler.projects.locations.jobs.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1alpha1/{+name}',
        request_field='job',
        request_type_name='CloudschedulerProjectsLocationsJobsPatchRequest',
        response_type_name='Job',
        supports_download=False,
    )

    def Pause(self, request, global_params=None):
      r"""Pauses a job. If a job is paused then the system will stop executing the job until it is re-enabled via CloudScheduler.ResumeJob. The state of the job is stored in Job.state; if paused it will be set to Job.State.PAUSED. A job must be in Job.State.ENABLED to be paused.

      Args:
        request: (CloudschedulerProjectsLocationsJobsPauseRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Job) The response message.
      """
      config = self.GetMethodConfig('Pause')
      return self._RunMethod(
          config, request, global_params=global_params)

    Pause.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/projects/{projectsId}/locations/{locationsId}/jobs/{jobsId}:pause',
        http_method='POST',
        method_id='cloudscheduler.projects.locations.jobs.pause',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha1/{+name}:pause',
        request_field='pauseJobRequest',
        request_type_name='CloudschedulerProjectsLocationsJobsPauseRequest',
        response_type_name='Job',
        supports_download=False,
    )

    def Resume(self, request, global_params=None):
      r"""Resume a job. This method reenables a job after it has been Job.State.PAUSED. The state of a job is stored in Job.state; after calling this method it will be set to Job.State.ENABLED. A job must be in Job.State.PAUSED to be resumed.

      Args:
        request: (CloudschedulerProjectsLocationsJobsResumeRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Job) The response message.
      """
      config = self.GetMethodConfig('Resume')
      return self._RunMethod(
          config, request, global_params=global_params)

    Resume.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/projects/{projectsId}/locations/{locationsId}/jobs/{jobsId}:resume',
        http_method='POST',
        method_id='cloudscheduler.projects.locations.jobs.resume',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha1/{+name}:resume',
        request_field='resumeJobRequest',
        request_type_name='CloudschedulerProjectsLocationsJobsResumeRequest',
        response_type_name='Job',
        supports_download=False,
    )

    def Run(self, request, global_params=None):
      r"""Forces a job to run now. When this method is called, Cloud Scheduler will dispatch the job, even if the job is already running.

      Args:
        request: (CloudschedulerProjectsLocationsJobsRunRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Job) The response message.
      """
      config = self.GetMethodConfig('Run')
      return self._RunMethod(
          config, request, global_params=global_params)

    Run.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/projects/{projectsId}/locations/{locationsId}/jobs/{jobsId}:run',
        http_method='POST',
        method_id='cloudscheduler.projects.locations.jobs.run',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha1/{+name}:run',
        request_field='runJobRequest',
        request_type_name='CloudschedulerProjectsLocationsJobsRunRequest',
        response_type_name='Job',
        supports_download=False,
    )

  class ProjectsLocationsService(base_api.BaseApiService):
    """Service class for the projects_locations resource."""

    _NAME = 'projects_locations'

    def __init__(self, client):
      super(CloudschedulerV1alpha1.ProjectsLocationsService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Gets information about a location.

      Args:
        request: (CloudschedulerProjectsLocationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Location) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/projects/{projectsId}/locations/{locationsId}',
        http_method='GET',
        method_id='cloudscheduler.projects.locations.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha1/{+name}',
        request_field='',
        request_type_name='CloudschedulerProjectsLocationsGetRequest',
        response_type_name='Location',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists information about the supported locations for this service.

      Args:
        request: (CloudschedulerProjectsLocationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListLocationsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/projects/{projectsId}/locations',
        http_method='GET',
        method_id='cloudscheduler.projects.locations.list',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['filter', 'pageSize', 'pageToken'],
        relative_path='v1alpha1/{+name}/locations',
        request_field='',
        request_type_name='CloudschedulerProjectsLocationsListRequest',
        response_type_name='ListLocationsResponse',
        supports_download=False,
    )

  class ProjectsService(base_api.BaseApiService):
    """Service class for the projects resource."""

    _NAME = 'projects'

    def __init__(self, client):
      super(CloudschedulerV1alpha1.ProjectsService, self).__init__(client)
      self._upload_configs = {
          }



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Task for rewriting an object's underlying data to update the metadata."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.storage import api_factory
from googlecloudsdk.api_lib.storage import request_config_factory
from googlecloudsdk.command_lib.storage import encryption_util
from googlecloudsdk.command_lib.storage import progress_callbacks
from googlecloudsdk.command_lib.storage.tasks import task
from googlecloudsdk.command_lib.storage.tasks.objects import patch_object_task
from googlecloudsdk.core import log


class RewriteObjectTask(task.Task):
  """Rewrites a cloud storage object's underlying data, changing metadata."""

  def __init__(self, object_resource, user_request_args=None):
    """Initializes task.

    Args:
      object_resource (resource_reference.ObjectResource): The object to update.
      user_request_args (UserRequestArgs|None): Describes metadata updates to
        perform.
    """
    super(RewriteObjectTask, self).__init__()
    self._object_resource = object_resource
    self._user_request_args = user_request_args

  def execute(self, task_status_queue=None):
    log.status.Print('Rewriting {}...'.format(self._object_resource))
    provider = self._object_resource.storage_url.scheme
    request_config = request_config_factory.get_request_config(
        self._object_resource.storage_url,
        user_request_args=self._user_request_args)

    api_client = api_factory.get_api(provider)
    existing_object_resource = api_client.get_object_metadata(
        self._object_resource.storage_url.bucket_name,
        self._object_resource.storage_url.object_name,
        generation=self._object_resource.storage_url.generation,
        request_config=request_config)

    if existing_object_resource.kms_key:  # Existing CMEK.
      encryption_changing = existing_object_resource.kms_key != getattr(
          encryption_util.get_encryption_key(), 'key', None)
    elif existing_object_resource.decryption_key_hash:  # Existing CSEK.
      encryption_changing = (
          existing_object_resource.decryption_key_hash != getattr(
              encryption_util.get_encryption_key(), 'sha256', None))
    else:  # No existing encryption.
      # Clear flag can still reset an object to bucket's default encryption.
      encryption_changing = encryption_util.get_encryption_key() is not None

    new_storage_class = getattr(request_config.resource_args, 'storage_class',
                                None)
    storage_class_changing = (
        new_storage_class and
        new_storage_class != existing_object_resource.storage_class)

    if not (encryption_changing or storage_class_changing):
      log.warning('Proposed encryption key and storage class for' +
                  ' {} match the existing data.'.format(self._object_resource) +
                  ' Performing patch instead of rewrite.')
      return task.Output(
          additional_task_iterators=[[
              patch_object_task.PatchObjectTask(self._object_resource,
                                                self._user_request_args)
          ]],
          messages=None)

    if storage_class_changing and not encryption_changing:
      # Preserve current encryption.
      new_encryption_key = encryption_util.get_encryption_key(
          existing_object_resource.decryption_key_hash,
          self._object_resource.storage_url)
    else:
      new_encryption_key = encryption_util.get_encryption_key()

    request_config_with_encryption = request_config_factory.get_request_config(
        self._object_resource.storage_url,
        user_request_args=self._user_request_args,
        decryption_key_hash=existing_object_resource.decryption_key_hash,
        encryption_key=new_encryption_key)

    api_client.copy_object(
        existing_object_resource,
        self._object_resource,
        request_config_with_encryption,
        should_deep_copy_metadata=True)

    if task_status_queue:
      progress_callbacks.increment_count_callback(task_status_queue)

  def __eq__(self, other):
    if not isinstance(other, type(self)):
      return NotImplemented
    return (self._object_resource == other._object_resource and
            self._user_request_args == other._user_request_args)
